#!/usr/bin/python
"""
wxmplot PlotPanel: a wx.Panel for line plotting, using matplotlib
"""
import os
import sys
from functools import partial
from datetime import datetime

import wx

from numpy import nonzero, where
import matplotlib as mpl
from matplotlib.dates import date2num, datestr2num

from matplotlib.figure import Figure
from matplotlib.ticker import FuncFormatter
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.gridspec import GridSpec
from matplotlib.colors import colorConverter
from matplotlib.collections import CircleCollection

from .basepanel import BasePanel
from .config import PlotConfig, ifnot_none
from .utils import inside_poly, fix_filename, gformat, MenuItem
from .plotconfigframe import PlotConfigFrame

to_rgba = colorConverter.to_rgba

class PlotPanel(BasePanel):
    """
    MatPlotlib line plot as a wx.Panel, suitable for embedding
    in any wx.Frame.   This does provide a right-click popup
    menu for configuration, zooming, saving an image of the
    figure, and Ctrl-C for copy-image-to-clipboard.

    For more features, see PlotFrame, which embeds a PlotPanel
    and also provides, a Menu, StatusBar, and Printing support.
    """

    def __init__(self, parent, size=(700, 450), dpi=150, axisbg=None,
                 facecolor=None, fontsize=9, trace_color_callback=None,
                 output_title='plot', with_data_process=True, theme=None,
                 **kws):

        self.trace_color_callback = trace_color_callback
        BasePanel.__init__(self, parent,
                           output_title=output_title, size=size, **kws)

        self.conf = PlotConfig(panel=self, theme=theme,
                               with_data_process=with_data_process)
        self.data_range = {}
        self.win_config = None
        self.cursor_callback = None
        self.lasso_callback = None
        self.cursor_mode = 'zoom'
        self.parent  = parent
        self.figsize = (size[0]*1.0/dpi, size[1]*1.0/dpi)
        self.dpi  = dpi
        self.conf.facecolor = ifnot_none(axisbg, self.conf.facecolor)
        self.conf.facecolor = ifnot_none(facecolor, self.conf.facecolor)

        # axesmargins : margins in px left/top/right/bottom
        self.axesmargins = (30, 30, 30, 30)

        self.BuildPanel()
        self.conf.user_limits = {} # [None, None, None, None]
        self.data_range = {}
        self.conf.zoom_lims = []
        self.conf.axes_traces = {}
        self.use_dates = False
        self.dates_style = None

    def plot(self, xdata, ydata, side='left', title=None,
             xlabel=None, ylabel=None, y2label=None,
             use_dates=False, dates_style=None, **kws):
        """
        create a new plot of x/y data, clearing any existing plot on the panel

        """
        allaxes = self.fig.get_axes()
        if len(allaxes) > 1:
            for ax in allaxes[1:]:
                if ax in self.data_range:
                    self.data_range.pop(ax)
                self.fig.delaxes(ax)

        self.data_range = {}
        self.conf.zoom_lims = []
        self.conf.axes_traces = {}
        self.clear()
        axes = self.axes
        if side == 'right':
            axes = self.get_right_axes()
        self.conf.reset_lines()
        self.conf.yscale = 'linear'
        self.conf.user_limits[axes] = 4*[None]

        if xlabel is not None:
            self.set_xlabel(xlabel, delay_draw=True)
        if ylabel is not None:
            self.set_ylabel(ylabel, delay_draw=True)
        if y2label is not None:
            self.set_y2label(y2label, delay_draw=True)
        if title is not None:
            self.set_title(title, delay_draw=True)
        self.dates_style = ifnot_none(dates_style, self.dates_style)
        self.use_dates = ifnot_none(use_dates, self.use_dates)
        return self.oplot(xdata, ydata, side=side, **kws)


    def oplot(self, xdata, ydata, side='left', label=None, xlabel=None,
              ylabel=None, y2label=None, title=None, dy=None,
              ylog_scale=None, xlog_scale=None, grid=None, xmin=None,
              xmax=None, ymin=None, ymax=None, color=None, style=None, alpha=None,
              fill=False, drawstyle=None, linewidth=2,
              marker=None, markersize=None, refresh=True, show_legend=None,
              legend_loc='best', legend_on=True, delay_draw=False,
              bgcolor=None, framecolor=None, gridcolor=None, textcolor=None,
              labelfontsize=None, titlefontsize=None, legendfontsize=None,
              fullbox=None, axes_style=None, zorder=None, viewpad=None,
              theme=None, use_dates=None, dates_style=None, **kws):
        """
        basic plot method, adding to an existing display

        """

        self.cursor_mode = 'zoom'
        conf = self.conf
        conf.plot_type = 'lineplot'
        axes = self.axes

        if theme is not None:
            conf.set_theme(theme=theme)
        if side == 'right':
            axes = self.get_right_axes()
        # set y scale to log/linear
        if ylog_scale is not None:
            conf.yscale = {False:'linear', True:'log'}[ylog_scale]

        if xlog_scale is not None:
            conf.xscale = {False:'linear', True:'log'}[xlog_scale]

        axes.xaxis.set_major_formatter(FuncFormatter(self.xformatter))

        self.dates_style = ifnot_none(dates_style, self.dates_style)
        self.use_dates = ifnot_none(use_dates, self.use_dates)
        if isinstance(xdata[0], datetime):
            self.use_dates = True

        if self.use_dates:
            # date handling options to get xdate to mpl dates
            #   1. xdate are in datetime: convert to mpl dates
            #   2. xdata are strings: parse with datestr2num
            #   3. xdata are floats: convert as unix timestamp to mpl dates
            x0 = xdata[0]
            dstyle = self.dates_style
            if dstyle is None:
                dstyle = ''
            if isinstance(x0, datetime):
                xdata = date2num(xdata)
            elif isinstance(x0, str) or dstyle.lower().startswith('str'):
                xdata = datestr2num(xdata)

        linewidth = ifnot_none(linewidth, 2)
        conf.viewpad = ifnot_none(viewpad, conf.viewpad)

        if xlabel is not None:
            self.set_xlabel(xlabel, delay_draw=True)
        if ylabel is not None:
            self.set_ylabel(ylabel, delay_draw=True)
        if y2label is not None:
            self.set_y2label(y2label, delay_draw=True)
        if title  is not None:
            self.set_title(title, delay_draw=True)
        if show_legend is not None:
            conf.set_legend_location(legend_loc, legend_on)
            conf.show_legend = show_legend

        conf.show_grid = ifnot_none(grid, conf.show_grid)

        # set data range for this trace
        # datarange = [min(xdata), max(xdata), min(ydata), max(ydata)]
        if axes not in conf.user_limits:
            conf.user_limits[axes] = [None, None, None, None]

        conf.user_limits[axes][0] = ifnot_none(xmin, conf.user_limits[axes][0])
        conf.user_limits[axes][1] = ifnot_none(xmax, conf.user_limits[axes][1])
        conf.user_limits[axes][2] = ifnot_none(ymin, conf.user_limits[axes][2])
        conf.user_limits[axes][3] = ifnot_none(ymax, conf.user_limits[axes][3])

        if axes == self.axes:
            axes.yaxis.set_major_formatter(FuncFormatter(self.yformatter))
        else:
            axes.yaxis.set_major_formatter(FuncFormatter(self.y2formatter))

        zorder = ifnot_none(zorder, 5*(conf.ntrace+1))

        if axes not in conf.axes_traces:
            conf.axes_traces[axes] = []
        conf.axes_traces[axes].append(conf.ntrace)

        conf.gridcolor = ifnot_none(gridcolor, conf.gridcolor)
        conf.set_gridcolor(conf.gridcolor)
        conf.facecolor = ifnot_none(bgcolor, conf.facecolor)
        conf.set_facecolor(conf.facecolor)
        conf.textcolor = ifnot_none(textcolor, conf.textcolor)
        conf.set_textcolor(conf.textcolor, delay_draw=True)
        if framecolor is not None:
            conf.set_framecolor(framecolor)

        conf.set_trace_zorder(zorder, delay_draw=True)
        if color:
            conf.set_trace_color(color, delay_draw=True)
        if style:
            conf.set_trace_style(style, delay_draw=True)
        if marker:
            conf.set_trace_marker(marker, delay_draw=True)
        if linewidth is not None:
            conf.set_trace_linewidth(linewidth, delay_draw=True)
        if markersize is not None:
            conf.set_trace_markersize(markersize, delay_draw=True)
        if drawstyle is not None:
            conf.set_trace_drawstyle(drawstyle, delay_draw=True)
        if alpha is not None:
            conf.set_trace_alpha(alpha, delay_draw=True)

        conf.dy[conf.ntrace] = dy
        if fill:
            fkws = dict(step=None, zorder=zorder, color=color)
            if drawstyle != 'default':
                fkws['step'] = drawstyle
            _lines = axes.plot(xdata, ydata, drawstyle=drawstyle, zorder=zorder, color=color)
            if dy is None:
                _fill = axes.fill_between(xdata, ydata, y2=0, **fkws)
            else:
                _fill = axes.fill_between(xdata, ydata-dy, y2=ydata+dy, **fkws)
        else: # not filling -- most plots here
            _fill = None
            if dy is not None:
                ebar = axes.errorbar(xdata, ydata, yerr=dy, zorder=zorder)
                _lines = [ebar.lines[0], ebar.lines[2]]
            else:
                _lines = axes.plot(xdata, ydata, drawstyle=drawstyle, zorder=zorder)

        conf.traces[conf.ntrace].fill = fill

        if axes not in conf.data_save:
            conf.data_save[axes] = []
        conf.data_save[axes].append((xdata, ydata))

        if conf.show_grid and axes == self.axes:
            # I'm sure there's a better way...
            # conf.set_gridcolor(conf.gridcolor)
            mpl.rcParams['grid.color'] = conf.gridcolor

            # grid_color = mpl.rcParams["grid.color"]
            # for i in axes.get_xgridlines() + axes.get_ygridlines():
            #     i.set_color(conf.gridcolor)
            #     i.set_zorder(-100)
            axes.grid(True)
        else:
            axes.grid(False)

        if (self.conf.xscale == 'log' or self.conf.yscale == 'log'):
            self.set_logscale(xscale=self.conf.xscale,
                              yscale=self.conf.yscale,
                              delay_draw=True)

        if label is None:
            label = 'trace %i' % (conf.ntrace+1)
        conf.set_trace_label(label, delay_draw=True)
        needs_relabel = False
        if labelfontsize is not None:
            conf.labelfont.set_size(labelfontsize)
            needs_relabel = True
        if titlefontsize is not None:
            conf.titlefont.set_size(titlefontsize)
            needs_relabel = True

        if legendfontsize is not None:
            conf.legendfont.set_size(legendfontsize)
            needs_relabel = True

        if conf.ntrace < len(conf.lines):
            conf.lines[conf.ntrace] = _lines
            conf.fills[conf.ntrace] = _fill
        else:
            conf.init_trace(conf.ntrace, 'black', 'solid')
            conf.lines.append(_lines)
            conf.fills.append(_fill)

        # now set plot limits:
        if not delay_draw:
            self.set_viewlimits()

        if refresh:
            conf.refresh_trace(conf.ntrace)
            needs_relabel = True
        if conf.show_legend:
            conf.draw_legend(delay_draw=True)

        if needs_relabel:
            conf.relabel(delay_draw=True)

        # axes style ('box' or 'open')
        conf.axes_style = 'box'
        if fullbox is not None and not fullbox:
            conf.axes_style = 'open'
        if axes_style in ('open', 'box', 'bottom'):
            conf.axes_style = axes_style

        conf.set_axes_style(delay_draw=delay_draw)
        if not delay_draw:
            self.draw()
            # self.canvas.Refresh()
        conf.ntrace = conf.ntrace + 1
        return _lines

    def plot_many(self, datalist, side='left', title=None,
                  xlabel=None, ylabel=None, show_legend=False, zoom_limits=None, **kws):
        """
        plot many traces at once, taking a list of (x, y) pairs
        """
        def unpack_tracedata(tdat, **kws):
            if (isinstance(tdat, dict) and
                'xdata' in tdat and 'ydata' in tdat):
                xdata = tdat.pop('xdata')
                ydata = tdat.pop('ydata')
                out = kws
                out.update(tdat)
            elif isinstance(tdat, (list, tuple)):
                out = kws
                xdata = tdat[0]
                ydata = tdat[1]
            return (xdata, ydata, out)


        conf = self.conf
        opts = dict(side=side, title=title, xlabel=xlabel, ylabel=ylabel,
                    delay_draw=True, show_legend=False)
        opts.update(kws)
        x0, y0, opts = unpack_tracedata(datalist[0], **opts)

        nplot_traces = len(conf.traces)
        nplot_request = len(datalist)
        if nplot_request > nplot_traces:
            linecolors = conf.linecolors
            ncols = len(linecolors)
            for i in range(nplot_traces, nplot_request+5):
                conf.init_trace(i,  linecolors[i%ncols], 'dashed')

        self.plot(x0, y0, **opts)
        for dat in datalist[1:]:
            x, y, opts = unpack_tracedata(dat, delay_draw=True)
            self.oplot(x, y, **opts)

        self.reset_formats()
        self.set_zoomlimits(zoom_limits)
        self.conf.show_legend = show_legend
        if show_legend:
            conf.draw_legend(delay_draw=True)
        conf.relabel(delay_draw=True)
        self.draw()
        # self.canvas.Refresh()


    def get_zoomlimits(self):
        return self.axes, self.get_viewlimits(), self.conf.zoom_lims

    def set_zoomlimits(self, limits):
        """set zoom limits returned from get_zoomlimits()"""
        if limits is None:
            # print("panel.set_zoom none")
            return False
        ax, vlims, zoom_lims = limits
        if ax == self.axes:
            try:
                ax.set_xlim((vlims[0], vlims[1]), emit=True)
                ax.set_ylim((vlims[2], vlims[3]), emit=True)
                if len(zoom_lims) > 0:
                    self.conf.zoom_lims = zoom_lims
            except:
                # print("panel.set_zoom error")
                return False
        return True


    def add_text(self, text, x, y, side='left', size=None,
                 rotation=None, ha='left', va='center',
                 family=None, **kws):
        """add text at supplied x, y position
        """
        axes = self.axes
        if side == 'right':
            axes = self.get_right_axes()
        dynamic_size = False
        if size is None:
            size = self.conf.legendfont.get_size()
            dynamic_size = True
        t = axes.text(x, y, text, ha=ha, va=va, size=size,
                      rotation=rotation, family=family, **kws)
        self.conf.added_texts.append((dynamic_size, t))
        self.draw()

    def add_arrow(self, x1, y1, x2, y2,  side='left',
                  shape='full', color='black',
                  width=0.01, head_width=0.03, overhang=0, **kws):
        """add arrow supplied x, y position"""
        dx, dy = x2-x1, y2-y1

        axes = self.axes
        if side == 'right':
            axes = self.get_right_axes()
        axes.arrow(x1, y1, dx, dy, shape=shape,
                   length_includes_head=True,
                   fc=color, edgecolor=color,
                   width=width, head_width=head_width,
                   overhang=overhang, **kws)
        self.draw()

    def scatterplot(self, xdata, ydata, label=None, size=10,
                    color=None, edgecolor=None,
                    selectcolor=None, selectedge=None,
                    xlabel=None, ylabel=None, y2label=None,
                    xmin=None, xmax=None, ymin=None, ymax=None,
                    viewpad=None, title=None, grid=None, callback=None, **kw):

        if xlabel is not None:
            self.set_xlabel(xlabel)
        if ylabel is not None:
            self.set_ylabel(ylabel)
        if y2label is not None:
            self.set_y2label(y2label)
        if title  is not None:
            self.set_title(title)
        if grid is not None:
            self.conf.show_grid = grid
        if callback is not None:
            self.lasso_callback = callback

        self.conf.plot_type = 'scatter'
        self.cursor_mode = 'lasso'
        if color is not None:
            self.conf.scatter_normalcolor = color
        if edgecolor is not None:
            self.conf.scatter_normaledge  = edgecolor
        if selectcolor is not None:
            self.conf.scatter_selectcolor = selectcolor
        if selectedge is not None:
            self.conf.scatter_selectedge = selectedge
        if viewpad is not None:
            self.conf.viewpad = viewpad

        axes = self.axes
        self.conf.user_limits[axes] = [xmin, xmax, ymin, ymax]

        self.conf.axes_traces = {axes: [0]}
        self.conf.set_trace_label('scatterplot')
        # self.conf.set_trace_datarange((min(xdata), max(xdata),
        #                                min(ydata), max(ydata)))

        self.conf.scatter_xdata = xdata
        self.conf.scatter_ydata = ydata
        self.axes.scatter(xdata, ydata, c=self.conf.scatter_normalcolor,
                          edgecolors=self.conf.scatter_normaledge)

        if self.conf.show_grid:
            for i in axes.get_xgridlines()+axes.get_ygridlines():
                i.set_color(self.conf.gridcolor)
                i.set_zorder(-30)
            axes.grid(True)
        else:
            axes.grid(False)
        xrange = max(xdata) - min(xdata)
        yrange = max(ydata) - min(ydata)

        xmin = min(xdata) - xrange/25.0
        xmax = max(xdata) + xrange/25.0
        ymin = min(ydata) - yrange/25.0
        ymax = max(ydata) + yrange/25.0

        axes.set_xlim((xmin, xmax), emit=True)
        axes.set_ylim((ymin, ymax), emit=True)
        self.set_viewlimits()
        self.draw()

    def lassoHandler(self, vertices):
        conf = self.conf
        if self.conf.plot_type == 'scatter':
            xd, yd = conf.scatter_xdata, conf.scatter_ydata
            sdat = list(zip(xd, yd))
            oldmask = conf.scatter_mask
            try:
                self.axes.scatter(xd[where(oldmask)], yd[where(oldmask)],
                                  s=conf.scatter_size,
                                  c=conf.scatter_normalcolor,
                                  edgecolors=conf.scatter_normaledge)
            except IndexError:
                self.axes.scatter(xd, yd, s=conf.scatter_size,
                                  c=conf.scatter_normalcolor,
                                  edgecolors=conf.scatter_normaledge)

            mask = conf.scatter_mask = inside_poly(vertices, sdat)
            pts = nonzero(mask)[0]
            self.axes.scatter(xd[where(mask)], yd[where(mask)],
                              s=conf.scatter_size,
                              c=conf.scatter_selectcolor,
                              edgecolors=conf.scatter_selectedge)

        else:
            xdata = self.axes.lines[0].get_xdata()
            ydata = self.axes.lines[0].get_ydata()
            sdat = [(x, y) for x, y in zip(xdata, ydata)]
            mask = inside_poly(vertices,sdat)
            pts = nonzero(mask)[0]

        self.lasso = None
        self.draw()
        # self.canvas.draw_idle()
        if (self.lasso_callback is not None and
            hasattr(self.lasso_callback , '__call__')):
            self.lasso_callback(data = sdat,
                                selected=pts, mask=mask)

    def set_xylims(self, limits, axes=None, side='left'):
        "set user-defined limits and apply them"
        if axes is None:
            axes = self.axes
            if side == 'right':
                axes = self.get_right_axes()
        self.conf.user_limits[axes] = list(limits)
        self.unzoom_all()

    def set_viewlimits(self):
        """updates xy limits of a plot based on current data,
        user defined limits, and any zoom level

        """
        self.reset_formats()
        self.conf.set_viewlimits()

    def get_viewlimits(self, axes=None):
        if axes is None: axes = self.axes
        xmin, xmax = axes.get_xlim()
        ymin, ymax = axes.get_ylim()
        return (xmin, xmax, ymin, ymax)

    def clear(self):
        """ clear plot """
        for ax in self.fig.get_axes():
            ax.cla()
        self.conf.ntrace = 0
        self.conf.xlabel = ''
        self.conf.ylabel = ''
        self.conf.y2label = ''
        self.conf.title  = ''
        self.conf.data_save = {}

    def reset_config(self):
        """reset configuration to defaults."""
        self.conf.set_defaults()

    def unzoom(self, event=None, **kws):
        """ zoom out 1 level, or to full data range """
        self.reset_formats()
        self.conf.unzoom(full=False)

    def unzoom_all(self, event=None):
        """ zoom out full data range """
        self.reset_formats()
        self.conf.unzoom(full=True)

    def process_data(self, event=None, expr=None):
        if expr in self.conf.data_expressions:
            self.conf.data_expr = expr
            self.conf.process_data()
            self.draw()
        if expr is None:
            expr = ''
        if self.conf.data_deriv:
            if expr is None:
                expr = 'y'
            expr = "deriv(%s)" % expr
        self.write_message("plotting %s" % expr, panel=0)

    def toggle_deriv(self, event=None, value=None):
        "toggle derivative of data"
        if value is None:
            self.conf.data_deriv = not self.conf.data_deriv

            expr = self.conf.data_expr or ''
            if self.conf.data_deriv:
                expr = "deriv(%s)" % expr
            self.write_message("plotting %s" % expr, panel=0)

            self.conf.process_data()

    def set_logscale(self, event=None, xscale='linear', yscale='linear',
                     delay_draw=False):
        "set log or linear scale for x, y axis"
        self.conf.set_logscale(xscale=xscale, yscale=yscale,
                               delay_draw=delay_draw)

    def toggle_legend(self, evt=None, show=None):
        "toggle legend display"
        if show is None:
            show = not self.conf.show_legend
            self.conf.show_legend = show
        self.conf.draw_legend()

    def toggle_grid(self, event=None, show=None):
        "toggle grid display"
        if show is None:
            show = not self.conf.show_grid
        self.conf.enable_grid(show)

    def configure(self, event=None):
        """show configuration frame"""
        if self.win_config is not None:
            try:
                self.win_config.Raise()
            except:
                self.win_config = None

        if self.win_config is None:
            self.win_config = PlotConfigFrame(parent=self,
                                              config=self.conf,
                                              trace_color_callback=self.trace_color_callback)
            self.win_config.Raise()

    ####
    ## create GUI
    ####x
    def BuildPanel(self):
        """ builds basic GUI panel and popup menu"""
        self.fig   = Figure(self.figsize, dpi=self.dpi)
        # 1 axes for now
        self.gridspec = GridSpec(1,1)
        self.axes  = self.fig.add_subplot(self.gridspec[0],
                                          facecolor=self.conf.facecolor)
        self.canvas = FigureCanvas(self, -1, self.fig)
        self.canvas.SetClientSize((self.figsize[0]*self.dpi, self.figsize[1]*self.dpi))
        self.canvas.SetMinSize((100, 100))
        self.canvas.gui_repaint = self.gui_repaint

        self.printer.canvas = self.canvas
        self.set_bg(self.conf.framecolor)
        self.conf.canvas = self.canvas
        self.canvas.SetCursor(wx.Cursor(wx.CURSOR_CROSS))
        self.canvas.mpl_connect("pick_event", self.__onPickEvent)

        # overwrite ScalarFormatter from ticker.py here:
        self.axes.xaxis.set_major_formatter(FuncFormatter(self.xformatter))
        self.axes.yaxis.set_major_formatter(FuncFormatter(self.yformatter))

        # This way of adding to sizer allows resizing
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.canvas, 2, wx.LEFT|wx.TOP|wx.BOTTOM|wx.EXPAND, 0)
        # self.SetAutoLayout(True)
        self.autoset_margins()
        self.SetSizer(sizer)
        self.SetSize(self.GetBestVirtualSize())

        canvas_draw = self.canvas.draw
        def draw(*args, **kws):
            self.autoset_margins()
            canvas_draw(*args, **kws)
        self.canvas.draw = draw
        self.addCanvasEvents()

    def BuildPopup(self):
        # build pop-up menu for right-click display
        self.popup_menu = popup = wx.Menu()
        MenuItem(self, popup, 'Configure', '',   self.configure)

        MenuItem(self, popup, 'Save Image', '',   self.save_figure)
        popup.AppendSeparator()

        MenuItem(self, popup, 'Undo Zoom/Pan', '',   self.unzoom)
        MenuItem(self, popup, 'Zoom all the way out', '', self.unzoom_all)

        popup.AppendSeparator()
        MenuItem(self, popup, 'Zoom X and Y', '',
                 partial(self.onZoomStyle, style='both x and y'),
                 kind=wx.ITEM_RADIO, checked=True)
        MenuItem(self, popup, 'Zoom X Only', '',
                 partial(self.onZoomStyle, style='x only'),
                 kind=wx.ITEM_RADIO)
        MenuItem(self, popup, 'Zoom Y Only', '',
                 partial(self.onZoomStyle, style='y only'),
                 kind=wx.ITEM_RADIO)

    def onZoomStyle(self, event=None, style='both x and y'):
        self.conf.zoom_style = style

    def _updateCanvasDraw(self):
        """ Overload of the draw function that update
        axes position before each draw"""
        fn = self.canvas.draw
        def draw2(*a,**k):
            self._updateGridSpec()
            return fn(*a,**k)
        self.canvas.draw = draw2

    def get_default_margins(self):
        """get default margins"""
        trans = self.fig.transFigure.inverted().transform

        # Static margins
        l, t, r, b = self.axesmargins
        (l, b), (r, t) = trans(((l, b), (r, t)))

        # Extent
        dl, dt, dr, db = 0, 0, 0, 0
        for ax in self.fig.get_axes():
            (x0, y0),(x1, y1) = ax.get_position().get_points()
            try:
                (ox0, oy0), (ox1, oy1) = ax.get_tightbbox(self.canvas.get_renderer()).get_points()
                (ox0, oy0), (ox1, oy1) = trans(((ox0 ,oy0),(ox1 ,oy1)))
                dl = min(0.2, max(dl, (x0 - ox0)))
                dt = min(0.2, max(dt, (oy1 - y1)))
                dr = min(0.2, max(dr, (ox1 - x1)))
                db = min(0.2, max(db, (y0 - oy0)))
            except:
                pass

        return (l + dl, t + dt, r + dr, b + db)

    def autoset_margins(self):
        """auto-set margins  left, bottom, right, top
        according to the specified margins (in pixels)
        and axes extent (taking into account labels,
        title, axis)
        """
        if not self.conf.auto_margins:
            return

        self.conf.margins = l, t, r, b = self.get_default_margins()
        self.gridspec.update(left=l, top=1-t, right=1-r, bottom=b)
        # Axes positions update
        for ax in self.fig.get_axes():
            figpos = ax.get_subplotspec().get_position(self.canvas.figure)
            ax.set_position(figpos)

    def draw(self):
        self.canvas.draw()

    def update_line(self, trace, xdata, ydata, side='left', draw=False,
                    update_limits=True):
        """ update a single trace, for faster redraw """
        x = self.conf.get_mpl_line(trace)
        if trace >= self.conf.ntrace:
            self.oplot(xdata, ydata, side=side, delay_draw=True)
        x.set_data(xdata, ydata)
        axes = self.axes
        if side == 'right':
            axes = self.get_right_axes()

        if update_limits:
            self.set_viewlimits()
        if draw:
            self.draw()

    def get_figure(self):
        return self.fig

    def __onPickEvent(self, event=None):
        """pick events"""
        legline = event.artist
        legtrace = self.conf.legend_map.get(legline, None)
        visible = True
        if legtrace is not None and self.conf.hidewith_legend:
            line, trace, legline, legtext = legtrace
            visible = not line.get_visible()
            line.set_visible(visible)
            if self.conf.fills[trace] is not None:
                self.conf.fills[trace].set_visible(visible)

            if visible:
                legline.set_zorder(10.00)
                legline.set_alpha(1.00)
                legtext.set_zorder(10.00)
                legtext.set_alpha(1.00)
            else:
                legline.set_alpha(0.50)
                legtext.set_alpha(0.50)


    ####
    ## GUI events
    ####
    def report_leftdown(self, event=None):
        if event is None:
            return
        ex, ey = event.x, event.y
        msg = ''
        try:
            x, y = self.axes.transData.inverted().transform((ex, ey))
        except:
            x, y = event.xdata, event.ydata

        if x is not None and y is not None:
            msg = "X,Y= %g, %g" % (x, y)
        if len(self.fig.get_axes()) > 1:
            ax2 = self.fig.get_axes()[1]
            try:
                x2, y2 = ax2.transData.inverted().transform((ex, ey))
                msg = "X,Y,Y2= %g, %g, %g" % (x, y, y2)
            except:
                pass

        nsbar = getattr(self, 'nstatusbar', 1)
        self.write_message(msg,  panel=max(0, nsbar - 2))
        if (self.cursor_callback is not None and
            hasattr(self.cursor_callback , '__call__')):
            self.cursor_callback(x=event.xdata, y=event.ydata)



#-----------------------------------------------------------------------------
#  adopted from IPython.lib.external._nope.py from IPython
#     [ Copyright (C) 2013 Min RK
#       Distributed under the terms of the 2-clause BSD License.
#     ]
#-----------------------------------------------------------------------------

from contextlib import contextmanager

import ctypes
import ctypes.util

void_p = ctypes.c_void_p
uint64 = ctypes.c_uint64

objc = ctypes.cdll.LoadLibrary(ctypes.util.find_library('objc'))

objc.objc_getClass.restype = void_p
objc.sel_registerName.restype = void_p
objc.objc_msgSend.restype = void_p
objc.objc_msgSend.argtypes = [void_p, void_p]

msgSend = objc.objc_msgSend

def as_utf8(s):
    """ensure utf8 bytes"""
    if not isinstance(s, bytes):
        s = s.encode('utf8')
    return s

def SelName(name):
    """create a selector name (for methods)"""
    return objc.sel_registerName(as_utf8(name))

def GetClass(classname):
    """get an ObjC Class by name"""
    return objc.objc_getClass(as_utf8(classname))

# constants from Foundation

NSActivityIdleDisplaySleepDisabled             = (1 << 40)
NSActivityIdleSystemSleepDisabled              = (1 << 20)
NSActivitySuddenTerminationDisabled            = (1 << 14)
NSActivityAutomaticTerminationDisabled         = (1 << 15)
NSActivityUserInitiated                        = (0x00FFFFFF | NSActivityIdleSystemSleepDisabled)
NSActivityUserInitiatedAllowingIdleSystemSleep = (NSActivityUserInitiated & ~NSActivityIdleSystemSleepDisabled)
NSActivityBackground                           = 0x000000FF
NSActivityLatencyCritical                      = 0xFF00000000

_activity = None

def allow_idle():
    """disable App Nap by setting NSActivityUserInitiatedAllowingIdleSystemSleep"""
    global _activity

    reason = msgSend(GetClass('NSString'),
                     SelName("stringWithUTF8String:"),
                     as_utf8('reason'))
    info   = msgSend(GetClass('NSProcessInfo'),
                     SelName('processInfo'))
    options = uint64(NSActivityUserInitiatedAllowingIdleSystemSleep)
    _activity = msgSend(info,
                        SelName('beginActivityWithOptions:reason:'),
                        options, void_p(reason))



#!/usr/bin/env python
#
"""
This is a collection of general purpose utility functions and classes,
especially useful for wx functionality
"""
import sys
import wx
import wx.lib.masked as masked

import numpy


def set_float(val):
    """ utility to set a floating value,
    useful for converting from strings """
    out = None
    if not val in (None, ''):
        try:
            out = float(val)
        except ValueError:
            return None
        if numpy.isnan(out):
            out = default
    return out

class Closure:
    """A very simple callback class to emulate a closure (reference to
    a function with arguments) in python.

    This class holds a user-defined function to be executed when the
    class is invoked as a function.  This is useful in many situations,
    especially for 'callbacks' where lambda's are quite enough.
    Many Tkinter 'actions' can use such callbacks.

    >>>def my_action(x=None):
    ...    print('my action: x = ', x)
    >>>c = Closure(my_action,x=1)
    ..... sometime later ...
    >>>c()
     my action: x = 1
    >>>c(x=2)
     my action: x = 2

    based on Command class from J. Grayson's Tkinter book.
    """
    def __init__(self, func=None, *args, **kws):
        self.func  = func
        self.kws   = kws
        self.args  = args

    def __call__(self,  *args, **kws):
        self.kws.update(kws)
        if hasattr(self.func, '__call__'):
            self.args = args
            return self.func(*self.args, **self.kws)


class FloatCtrl(wx.TextCtrl):
    """ Numerical Float Control::
    a wx.TextCtrl that allows only numerical input, can take a precision argument
    and optional upper / lower bounds
    Options:

    """
    def __init__(self, parent, value='', minval=None, maxval=None,
                 precision=3, bell_on_invalid = True,
                 action=None, action_kw=None, **kws):

        self.__digits = '0123456789.-'
        self.__prec   = precision
        if precision is None:
            self.__prec = 0
        self.format   = '%%.%if' % self.__prec
        self.is_valid = True
        self.__val = set_float(value)
        self.__max = set_float(maxval)
        self.__min = set_float(minval)
        self.__bound_val = None
        self.__mark = None
        self.__action = None

        self.fgcol_valid   = "Black"
        self.bgcol_valid   = "White"
        self.fgcol_invalid = "Red"
        self.bgcol_invalid = (254, 254, 80)
        self.bell_on_invalid = bell_on_invalid

        # set up action
        if action_kw is None:
            action_kw = {}
        self.SetAction(action, **action_kw)

        this_sty =  wx.TE_PROCESS_ENTER|wx.TE_RIGHT
        if 'style' in kws:
            this_sty = this_sty | kws['style']
        kws['style'] = this_sty

        wx.TextCtrl.__init__(self, parent, wx.ID_ANY, **kws)

        self.__CheckValid(self.__val)
        self.SetValue(self.__val)

        self.Bind(wx.EVT_CHAR, self.OnChar)
        self.Bind(wx.EVT_TEXT, self.OnText)

        self.Bind(wx.EVT_SET_FOCUS,  self.OnSetFocus)
        self.Bind(wx.EVT_KILL_FOCUS, self.OnKillFocus)
        self.__GetMark()

    def SetAction(self, action, **kws):
        "set callback action"
        if hasattr(action,'__call__'):
            self.__action = Closure(action, **kws)

    def SetPrecision(self, prec=0):
        "set precision"
        self.__prec = prec
        self.format = '%%.%if' % prec

    def __GetMark(self):
        " keep track of cursor position within text"
        try:
            self.__mark = min(wx.TextCtrl.GetSelection(self)[0],
                              len(wx.TextCtrl.GetValue(self).strip()))
        except:
            self.__mark = 0

    def __SetMark(self, mark=None):
        "set mark for later"
        if mark is None:
            mark = self.__mark
        self.SetSelection(mark, mark)

    def SetValue(self, value=None, act=True):
        " main method to set value "
        if value is None:
            value = wx.TextCtrl.GetValue(self).strip()
        self.__CheckValid(value)
        self.__GetMark()
        if value is not None:
            wx.TextCtrl.SetValue(self, self.format % set_float(value))

        if self.is_valid and hasattr(self.__action, '__call__') and act:
            self.__action(value=self.__val)
        elif not self.is_valid and self.bell_on_invalid:
            wx.Bell()

        self.__SetMark()

    def OnKillFocus(self, event):
        "focus lost"
        self.__GetMark()
        event.Skip()

    def OnSetFocus(self, event):
        "focus gained - resume editing from last mark point"
        self.__SetMark()
        event.Skip()

    def OnChar(self, event):
        """ on Character event"""
        key   = event.GetKeyCode()
        entry = wx.TextCtrl.GetValue(self).strip()
        pos   = wx.TextCtrl.GetSelection(self)
        # really, the order here is important:
        # 1. return sends to ValidateEntry
        if key == wx.WXK_RETURN:
            if not self.is_valid:
                wx.TextCtrl.SetValue(self, self.format % set_float(self.__bound_val))
            else:
                self.SetValue(entry)
            return

        # 2. other non-text characters are passed without change
        if (key < wx.WXK_SPACE or key == wx.WXK_DELETE or key > 255):
            event.Skip()
            return

        # 3. check for multiple '.' and out of place '-' signs and ignore these
        #    note that chr(key) will now work due to return at #2

        has_minus = '-' in entry
        ckey = chr(key)
        if ((ckey == '.' and (self.__prec == 0 or '.' in entry) ) or
            (ckey == '-' and (has_minus or  pos[0] != 0)) or
            (ckey != '-' and  has_minus and pos[0] == 0)):
            return
        # 4. allow digits, but not other characters
        if chr(key) in self.__digits:
            event.Skip()


    def OnText(self, event=None):
        "text event"
        try:
            if event.GetString() != '':
                self.__CheckValid(event.GetString())
        except:
            pass
        event.Skip()

    def GetValue(self):
        if self.__prec > 0:
            fmt = "{{:.{:d}f}}".format(self.__prec)
            return set_float(fmt.format(self.__val))
        else:
            return int(self.__val)

    def GetMin(self):
        "return min value"
        return self.__min

    def GetMax(self):
        "return max value"
        return self.__max

    def SetMin(self, val):
        "set min value"
        self.__min = set_float(val)

    def SetMax(self, val):
        "set max value"
        self.__max = set_float(val)

    def __CheckValid(self, value):
        "check for validity of value"
        val = self.__val
        self.is_valid = True
        try:
            val = set_float(value)
            if self.__min is not None and (val < self.__min):
                self.is_valid = False
                val = self.__min
            if self.__max is not None and (val > self.__max):
                self.is_valid = False
                val = self.__max
        except:
            self.is_valid = False
        self.__bound_val = self.__val = val
        fgcol, bgcol = self.fgcol_valid, self.bgcol_valid
        if not self.is_valid:
            fgcol, bgcol = self.fgcol_invalid, self.bgcol_invalid

        self.SetForegroundColour(fgcol)
        self.SetBackgroundColour(bgcol)
        self.Refresh()



#!/usr/bin/python
#

import sys
from math import log10

from matplotlib.path import Path

import wx
from wx.lib.agw import floatspin as fspin


def fix_filename(s):
    """fix string to be a 'good' filename.
    This may be a more restrictive than the OS, but
    avoids nasty cases."""
    badchars = ' <>:"\'\\\t\r\n/|?*!%$'
    t = s.translate(s.maketrans(badchars, '_'*len(badchars)))
    if t.count('.') > 1:
        for i in range(t.count('.') - 1):
            idot = t.find('.')
            t = "%s_%s" % (t[:idot], t[idot+1:])
    return t

def pack(window, sizer, expand=1.1):
    "simple wxPython pack function"
    tsize =  window.GetSize()
    msize =  window.GetMinSize()
    window.SetSizer(sizer)
    sizer.Fit(window)
    nsize = (10*int(expand*(max(msize[0], tsize[0])/10)),
             10*int(expand*(max(msize[1], tsize[1])/10.)))
    window.SetSize(nsize)


class SimpleText(wx.StaticText):
    "simple static text wrapper"
    def __init__(self, parent, label, minsize=None, font=None, colour=None,
                 bgcolour=None, style=wx.ALIGN_CENTRE, **kws):

        wx.StaticText.__init__(self, parent, -1, label=label, style=style,
                               **kws)
        if minsize is not None:
            self.SetMinSize(minsize)
        if font is not None:
            self.SetFont(font)
        if colour is not None:
            self.SetForegroundColour(colour)
        if bgcolour is not None:
            self.SetBackgroundColour(bgcolour)

def HLine(parent, size=(700, 3)):
    """Simple horizontal line
    h = HLine(parent, size=(700, 3)
    """
    return wx.StaticLine(parent, size=size, style=wx.LI_HORIZONTAL|wx.GROW)

def MenuItem(parent, menu, label='', longtext='', action=None, kind='normal',
             checked=False):
    """Add Item to a Menu, with action
    m = Menu(parent, menu, label, longtext, action=None, kind='normal')
    """
    kinds_map = {'normal': wx.ITEM_NORMAL,
                 'radio': wx.ITEM_RADIO,
                 'check': wx.ITEM_CHECK}
    menu_kind = wx.ITEM_NORMAL
    if kind in kinds_map.values():
        menu_kind = kind
    elif kind in kinds_map:
        menu_kind = kinds_map[kind]

    item = menu.Append(-1, label, longtext, kind=menu_kind)
    if menu_kind == wx.ITEM_CHECK and checked:
        item.Check(True)

    if callable(action):
        parent.Bind(wx.EVT_MENU, action, item)
    return item

class Check(wx.CheckBox):
    """Simple Checkbox
    c = Check(parent, default=True, label=None, **kws)
    kws passed to wx.CheckBox
    """
    def __init__(self, parent, label='', default=True, action=None, **kws):
        wx.CheckBox.__init__(self, parent, -1, label=label, **kws)
        self.SetValue({True: 1, False:0}[default])
        if action is not None:
            self.Bind(wx.EVT_CHECKBOX, action)

class Choice(wx.Choice):
    """Simple Choice with default and bound action
    c = Choice(panel, choices, default=0, action=None, **kws)
    """
    def __init__(self, parent, choices=None, default=0,
                 action=None, **kws):
        if choices is None:
            choices = []
        wx.Choice.__init__(self, parent, -1,  choices=choices, **kws)
        self.Select(default)
        self.Bind(wx.EVT_CHOICE, action)

    def SetChoices(self, choices):
        index = 0
        try:
            current = self.GetStringSelection()
            if current in choices:
                index = choices.index(current)
        except:
            pass
        self.Clear()
        self.AppendItems(choices)
        self.SetStringSelection(choices[index])


def FloatSpin(parent, value=0, action=None, tooltip=None,
                 size=(100, -1), digits=1, increment=1, **kws):
    """FloatSpin with action and tooltip"""
    if value is None:
        value = 0
    fs = fspin.FloatSpin(parent, -1, size=size, value=value,
                         digits=digits, increment=increment, **kws)
    if action is not None:
        fs.Bind(fspin.EVT_FLOATSPIN, action)
    if tooltip is not None:
        fs.SetToolTip(tooltip)
    return fs


def gformat(val, length=11):
    """Format a number with '%g'-like format, except that

        a) the length of the output string will be the requested length.
        b) positive numbers will have a leading blank.
        b) the precision will be as high as possible.
        c) trailing zeros will not be trimmed.

    The precision will typically be length-7.

    Arguments
    ---------
    val       value to be formatted
    length    length of output string

    Returns
    -------
    string of specified length.

    Notes
    ------
     Positive values will have leading blank.

    """
    if val is None or isinstance(val, bool):
        return f'{repr(val):>{length}s}'
    try:
        expon = int(log10(abs(val)))
    except (OverflowError, ValueError):
        expon = 0
    except TypeError:
        return f'{repr(val):>{length}s}'

    length = max(length, 7)
    form = 'e'
    prec = length - 7
    if abs(expon) > 99:
        prec -= 1
    elif ((expon > 0 and expon < (prec+4)) or
          (expon <= 0 and -expon < (prec-1))):
        form = 'f'
        prec += 4
        if expon > 0:
            prec -= expon
    return f'{val:{length}.{prec}{form}}'


class LabeledTextCtrl(wx.TextCtrl):
    """
    simple extension of TextCtrl.  Typical usage:

    entry = LabeledTextCtrl(self, -1, value='22',
                            color='black',
                            labeltext='X',labelbgcolor='green',
                            style=wx.ALIGN_LEFT|wx.ST_NO_AUTORESIZE)
    row  = wx.BoxSizer(wx.HORIZONTAL)
    row.Add(entry.label, 1,wx.ALIGN_LEFT|wx.EXPAND)
    row.Add(entry,    1,wx.ALIGN_LEFT|wx.EXPAND)
    """
    def __init__(self,parent,value,size=(-1, -1),
                 font=None, action=None,
                 bgcolor=None, color=None, style=None,
                 labeltext=None, labelsize=(-1, -1),
                 labelcolor=None, labelbgcolor=None):

        if style is None:
            style=wx.ALIGN_LEFT|wx.ST_NO_AUTORESIZE|wx.TE_PROCESS_ENTER
        if action is None:
            action = self.GetValue
        self.action = action

        if labeltext is not None:
            self.label = wx.StaticText(parent, -1, labeltext,
                                       size=labelsize, style=style)

            if labelcolor:
                self.label.SetForegroundColour(labelcolor)
            if labelbgcolor:
                self.label.SetBackgroundColour(labelbgcolor)
            if font is not None:
                self.label.SetFont(font)

        try:
            value = str(value)
        except:
            value = ' '

        wx.TextCtrl.__init__(self, parent, -1, value, size=size,
                             style=style)

        self.Bind(wx.EVT_TEXT_ENTER, self.__act)
        self.Bind(wx.EVT_KILL_FOCUS, self.__act)
        if font is not None:
            self.SetFont(font)
        if color:
            self.SetForegroundColour(color)
        if bgcolor:
            self.SetBackgroundColour(bgcolor)

    def __act(self,event=None):
        self.action(event=event)
        val = self.GetValue()
        event.Skip()
        return val



class PrintoutWx(wx.Printout):
    """Simple wrapper around wx Printout class -- all the real work
    here is scaling the matplotlib canvas bitmap to the current
    printer's definition.
    """
    _title_ = 'wxmplot'
    def __init__(self, canvas, width=6.0, margin=0.25,
                 title=None):
        if title is None:
            title = self._title_
        wx.Printout.__init__(self, title)
        self.canvas = canvas
        self.width  = width
        self.margin = margin


    def HasPage(self, page):
        return page <= 1

    def GetPageInfo(self):
        return (1, 1, 1, 1)

    def OnPrintPage(self, page):
        self.canvas.draw()

        ppw,pph = self.GetPPIPrinter()      # printer's pixels per in
        pgw,pgh = self.GetPageSizePixels()  # page size in pixels
        grw, grh = self.canvas.GetSize()
        dc      = self.GetDC()
        dcw,dch = dc.GetSize()

        # save current figure dpi resolution and bg color,
        # so that we can temporarily set them to the dpi of
        # the printer, and the bg color to white
        bgcolor   = self.canvas.figure.get_facecolor()
        fig_dpi   = self.canvas.figure.dpi

        # draw the bitmap, scaled appropriately
        vscale    = float(ppw) / fig_dpi

        # set figure resolution,bg color for printer
        self.canvas.figure.dpi = ppw
        self.canvas.figure.set_facecolor('#FFFFFF')
        try:
            self.canvas.bitmap.SetWidth(int(self.canvas.bitmap.GetWidth() * vscale))
            self.canvas.bitmap.SetHeight(int(self.canvas.bitmap.GetHeight()* vscale))
        except:
            pass

        # occasional crash here?
        try:
            self.canvas.draw()
        except:
            return

        # page may need additional scaling on preview
        page_scale = 1.0
        if self.IsPreview():
            page_scale = float(dcw)/pgw

        # get margin in pixels = (margin in in) * (pixels/in)
        top_margin  = int(self.margin * pph * page_scale)
        left_margin = int(self.margin * ppw * page_scale)

        # set scale so that width of output is self.width inches
        # (assuming grw is size of graph in inches....)
        user_scale = (self.width * fig_dpi * page_scale)/float(grw)

        dc.SetDeviceOrigin(left_margin,top_margin)
        dc.SetUserScale(user_scale,user_scale)

        # this cute little number avoid API inconsistencies in wx
        try:
            dc.DrawBitmap(self.canvas.bitmap, 0, 0)
        except:
            pass
        # restore original figure  resolution
        self.canvas.figure.set_facecolor(bgcolor)
        self.canvas.figure.dpi = fig_dpi
        self.canvas.draw()
        return True

class Printer:
    def __init__(self, parent, title=None,
                 canvas=None, width=6.0, margin=0.5):
        """initialize printer settings using wx methods"""

        self.parent = parent
        self.canvas = canvas
        self.pwidth = width
        self.pmargin= margin
        self.title = title
        self.printerData = wx.PrintData()
        self.printerData.SetPaperId(wx.PAPER_LETTER)
        self.printerData.SetPrintMode(wx.PRINT_MODE_PRINTER)
        self.printerPageData= wx.PageSetupDialogData()
        self.printerPageData.SetMarginBottomRight((25,25))
        self.printerPageData.SetMarginTopLeft((25,25))
        self.printerPageData.SetPrintData(self.printerData)

    def Setup(self, event=None):
        """set up figure for printing.  Using the standard wx Printer
        Setup Dialog. """

        if hasattr(self, 'printerData'):
            data = wx.PageSetupDialogData()
            data.SetPrintData(self.printerData)
        else:
            data = wx.PageSetupDialogData()
        data.SetMarginTopLeft( (15, 15) )
        data.SetMarginBottomRight( (15, 15) )

        dlg = wx.PageSetupDialog(None, data)

        if dlg.ShowModal() == wx.ID_OK:
            data = dlg.GetPageSetupData()
            tl = data.GetMarginTopLeft()
            br = data.GetMarginBottomRight()
        self.printerData = wx.PrintData(data.GetPrintData())
        dlg.Destroy()

    def Preview(self, title=None, event=None):
        """ generate Print Preview with wx Print mechanism"""
        if title is None:
            title = self.title
        if self.canvas is None:
            self.canvas = self.parent.canvas

        po1  = PrintoutWx(self.parent.canvas, title=title,
                          width=self.pwidth,   margin=self.pmargin)
        po2  = PrintoutWx(self.parent.canvas, title=title,
                          width=self.pwidth,   margin=self.pmargin)
        self.preview = wx.PrintPreview(po1, po2, self.printerData)

        if self.preview.IsOk():
            self.preview.SetZoom(85)
            frameInst= self.parent
            while not isinstance(frameInst, wx.Frame):
                frameInst= frameInst.GetParent()
            frame = wx.PreviewFrame(self.preview, frameInst, "Preview")
            frame.Initialize()
            frame.SetSize((850, 650))
            frame.Centre(wx.BOTH)
            frame.Show(True)

    def Print(self, title=None, event=None):
        """ Print figure using wx Print mechanism"""
        pdd = wx.PrintDialogData()
        pdd.SetPrintData(self.printerData)
        pdd.SetToPage(1)
        printer  = wx.Printer(pdd)
        if title is None:
            title = self.title

        printout = PrintoutWx(self.parent.canvas, title=title,
                              width=self.pwidth, margin=self.pmargin)
        print_ok = printer.Print(self.parent, printout, True)

        if not print_ok and not printer.GetLastError() == wx.PRINTER_CANCELLED:
            wx.MessageBox("""There was a problem printing.
            Perhaps your current printer is not set correctly?""",
                          "Printing", wx.OK)
        printout.Destroy()

def inside_poly(vertices,data):
    return Path(vertices).contains_points(data)



#!/usr/bin/python
#

import wx
import numpy as np
import wxmplot

x   = np.linspace(0, 5, 201)

traces = []

for i in range(20):
    traces.append((x, np.sqrt(x)*np.sin(0.25*(i+1)*x)*np.exp(-x/2)))


app = wx.App()

pframe = wxmplot.PlotFrame(output_title='Plot Multiple')
pframe.plot_many(traces, title='Test Plotting 20 traces with plot_many()',
                 xlabel=r'x (mm)')

pframe.write_message('WXMPlot PlotFrame example: Try Help->Quick Reference')
pframe.Show()
#
app.MainLoop()



"""
example showing display of R, G, B maps
"""
import wx
from numpy import exp, random, arange, outer, array
from wxmplot import ImageFrame

def gauss2d(x, y, x0, y0, sx, sy):
    return outer( exp( -(((y-y0)/float(sy))**2)/2),
                  exp( -(((x-x0)/float(sx))**2)/2) )


if __name__ == '__main__':
    app = wx.App()
    frame = ImageFrame(mode='rgb')
    ny, nx = 350, 400
    x = arange(nx)
    y = arange(ny)
    ox =  x / 100.0
    oy = -1 + y / 200.0
    red  = 0.02 * random.random(size=nx*ny).reshape(ny, nx)
    red =  red + (6.0*gauss2d(x, y, 90,   76,  5,  6) +
                  3.0*gauss2d(x, y, 165, 190,  70,  33) +
                  2.0*gauss2d(x, y, 180, 100,  12,  6))
    green  = 0.3 * random.random(size=nx*ny).reshape(ny, nx)
    green = green  + (5.0*gauss2d(x, y, 173,  98,  4,  9) +
                      3.2*gauss2d(x, y, 270, 230, 78, 63))

    blue = 0.1 * random.random(size=nx*ny).reshape(ny, nx)
    blue = blue + (2.9*gauss2d(x, y, 240, 265,  78,  23) +
                   3.5*gauss2d(x, y, 185,  95,  22, 11) +
                   7.0*gauss2d(x, y, 220,  310,  40,  133))

    dat = array([red, green, blue]).swapaxes(2, 0)
    frame.display(dat, x=ox, y=oy,
                  subtitles={'red':'Red Image', 'green': 'Green Blob', 'blue': 'other'})
    frame.Show()
    app.MainLoop()



#!/usr/bin/env python
"""
   wxmplot: wxPython plotting widgets using matplotlib.

   wxmplot provides advanced wxPython widgets for plotting and image
   display based on matplotlib. The plotting and image display wx Panels
   and Frames it provides are easy for the programmer to include and work
   with from wx programs.  More importantly, the widgets created by wxmplot
   give the end user a flexible set of tools for interacting with their
   data and customizing the plots and displays.  wxmplot panels are more
   interactive than typical displayss from matplotlib's pyplot module.

   version: 0.9.56
   last update: 2023-Jul-02
   License:  MIT
   Author:  Matthew Newville <newville@cars.uchicago.edu>
            Center for Advanced Radiation Sources,
            The University of Chicago

   the main widgets provided by wxmplot are:

      PlotPanel: wx.Panel for basic 2-D line plots (roughly matplotlib `plot`)

      PlotFrame: wx.Frame containing a PlotPanel

      ImagePanel: wx.Panel for image display (roughly matplotlib `imshow`)

      ImageFrame: wx.Frame containing ImagePanel
"""
import sys
import wx

from .version import version as __version__
from .plotpanel import PlotPanel
from .imagepanel import ImagePanel

from .baseframe import BaseFrame
from .plotframe import PlotFrame

from .imageframe import ImageFrame
from .multiframe import MultiPlotFrame
from .stackedplotframe import StackedPlotFrame
from .residualplotframe import ResidualPlotFrame
from .imagematrixframe import ImageMatrixFrame
from .plotapp  import PlotApp

if sys.platform.lower() == 'darwin':
    wx.PyApp.IsDisplayAvailable = lambda _: True



import time, os, sys
from random import shuffle

import numpy as np

import wx
from wxmplot import ImageFrame

class TestFrame(wx.Frame):
    def __init__(self, parent=None, *args,**kwds):
        kwds["style"] = wx.DEFAULT_FRAME_STYLE|wx.RESIZE_BORDER|wx.TAB_TRAVERSAL

        wx.Frame.__init__(self, parent, -1, '',
                         wx.DefaultPosition, wx.Size(-1,-1), **kwds)
        self.SetTitle(" WXMPlot Image Scroll")

        self.SetFont(wx.Font(12,wx.SWISS,wx.NORMAL,wx.BOLD,False))
        menu = wx.Menu()

        self.arrays = []
        self.imageframe  = None

        framesizer = wx.BoxSizer(wx.VERTICAL)

        panel      = wx.Panel(self, -1, size=(-1, -1))
        panelsizer = wx.BoxSizer(wx.VERTICAL)

        panelsizer.Add( wx.StaticText(panel, -1, 'Show Images'),
                        0, wx.LEFT|wx.EXPAND, 10)

        btn = wx.Button(panel, -1, 'start scrolling', size=(-1,-1))
        btn.Bind(wx.EVT_BUTTON,self.onScrollImages)

        btn2 = wx.Button(panel, -1, 'stop scrolling', size=(-1,-1))
        btn2.Bind(wx.EVT_BUTTON,self.onStop)

        self.msg = wx.StaticText(panel, label='000 images in 00.0000 seconds ',  size=(500, -1))

        panelsizer.Add(btn, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(btn2, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(self.msg, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)

        panel.SetSizer(panelsizer)
        panelsizer.Fit(panel)

        framesizer.Add(panel, 0, wx.EXPAND, 2)
        self.SetSizer(framesizer)
        framesizer.Fit(self)
        self.create_data()
        self.ShowImageFrame()
        self.count    = 0
        self.Bind(wx.EVT_TIMER, self.onTimer)
        self.timer = wx.Timer(self)
        self.Refresh()

    def create_data(self):
        nx = ny = 1001
        print("Creating 240 images..")
        for xoff in np.linspace(-2.5, 2.5, 20):
            y, x = np.mgrid[-10+xoff:10+xoff:nx*1j, -10:10:nx*1j]
            for xscale in (4.5, 5.0, 6.0):
                for yscale in (0.8, 1.0, 1.1, 1.2):
                    dat = np.sin(x*x/xscale + y*y/yscale)/(1 + (x+y)*(x+y))
                    dat += np.random.normal(scale=0.12, size=(nx, ny))
                    self.arrays.append(dat)
        shuffle(self.arrays)
        print("Built %d arrays, shape=%s" % (len(self.arrays), repr(dat.shape)))

    def ShowImageFrame(self):
        if self.imageframe is None:
            self.imageframe = ImageFrame(self)
        try:
            self.imageframe.Show()
        except RuntimeError:
            self.imageframe = Imageframe(self)
            self.imageframe.Show()

        self.imageframe.display(self.arrays[0])
        self.imageframe.Raise()

    def onTimer(self, event):
        self.count += 1
        message = " %d images in %.3f sec" % (self.count,
                                              time.time()-self.t0)
        self.msg.SetLabel(message)
        wx.CallAfter(self.imageframe.panel.update_image,
                     self.arrays[self.count % len(self.arrays)])

    def onScrollImages(self,event=None):
        self.ShowImageFrame()
        self.count = 0
        self.t0 = time.time()
        self.timer.Start(10)

    def onStop(self,event=None):
        self.timer.Stop()

if __name__ == '__main__':
    app = wx.App()
    f = TestFrame(None,-1)
    f.Show(True)
    app.MainLoop()



import numpy as np
from wxmplot.interactive import plot, set_data_generator

# generator of datasets
npts = 501
x = np.linspace(0, 50, npts)
datasets = ((np.cos(1.3*x) + np.sin(0.8*(x+nx/7)),
             np.cos(1.1*x) - np.sin(0.6*(x+nx/43)))
            for nx in range(npts))

def more_data():
    """yield next pair of datasets from data generator"""
    while True:
        try:
            ds = next(datasets)
            yield [(x, ds[0]), (x, ds[1])]
        except StopIteration:
            break

# set up an initial plot
plot(x, np.zeros(len(x)))

# now set data generator and wait
set_data_generator(more_data,  polltime=30)
print("consuming data from generator...")



#!/usr/bin/python
"""
wxmplot PlotFrame: a wx.Frame for line plotting, using matplotlib
"""

import os
from functools import partial

import wx
from wxutils import get_cwd

from . import __version__
from .plotpanel import PlotPanel
from .utils import MenuItem, fix_filename

class BaseFrame(wx.Frame):
    """
    wx.Frame with PlotPanel
    """
    help_msg =  """
Left-Click:   to display X,Y coordinates
Left-Drag:    to zoom in on plot region
Right-Click:  display popup menu with choices:
           Zoom out 1 level
           Zoom all the way out
           Configure
           Save Image

With a Plot Legend displayed, click on each label to toggle the display of that trace.

Key bindings (use 'Apple' for 'Ctrl' on MacOSX):

Ctrl-S:     Save plot image to PNG file
Ctrl-C:     Copy plot image to system clipboard
Ctrl-P:     Print plot image

Ctrl-D:     Export data to plain text file

Ctrl-L:     Toggle Display of Plot Legend
Ctrl-G:     Toggle Display of Grid

Ctrl-K:     Show Plot Configure Frame

Ctrl-Q:     Quit
"""

    about_msg =  """WXMPlot  version %s
Matt Newville <newville@cars.uchicago.edu>""" % __version__


    def __init__(self, parent=None, panel=None, title='', size=None,
                 exit_callback=None, user_menus=None, panelkws=None,
                 axisbg=None, output_title='Plot', dpi=150,
                 with_data_process=True, theme=None, **kws):
        if size is None:
            size = (700, 500)
        kws['style'] = wx.DEFAULT_FRAME_STYLE
        kws['size']  = size
        wx.Frame.__init__(self, parent, -1, title, **kws)

        self.SetMinSize((250, 250))
        self.output_title = output_title
        self.exit_callback = exit_callback
        self.parent = parent
        self.panel  = panel
        self.dpi    = dpi
        self.user_menus = user_menus
        self.with_data_process = with_data_process
        self.size = size
        self.panelkws = {'dpi': dpi}
        if panelkws is not None:
            self.panelkw.update(panelkws)

        self.theme = theme
        if axisbg is not None:
            self.panelkws['axisbg'] = axisbg

    def write_message(self, txt, panel=0):
        """write a message to the Status Bar"""
        self.SetStatusText(txt, panel)

    def set_xylims(self, limits, axes=None):
        """overwrite data for trace t """
        if self.panel is not None:
            self.panel.set_xylims(limits, axes=axes)

    def clear(self):
        """clear plot """
        if self.panel is not None:
            self.panel.clear()

    def unzoom_all(self, event=None):
        """zoom out full data range """
        if self.panel is not None:
            self.panel.unzoom_all(event=event)

    def unzoom(self, event=None):
        """zoom out 1 level, or to full data range """
        if self.panel is not None: self.panel.unzoom(event=event)

    def set_title(self, s):
        "set plot title"
        if self.panel is not None:
            self.panel.set_title(s)

    def set_xlabel(self, s):
        "set plot xlabel"
        if self.panel is not None: self.panel.set_xlabel(s)
        self.panel.canvas.draw()

    def set_ylabel(self, s):
        "set plot xlabel"
        if self.panel is not None: self.panel.set_ylabel(s)
        self.panel.canvas.draw()

    def save_figure(self,event=None, transparent=False, dpi=600):
        """ save figure image to file"""
        if self.panel is not None:
            self.panel.save_figure(event=event,
                                   transparent=transparent, dpi=dpi)

    def configure(self,event=None):
        if self.panel is not None: self.panel.configure(event=event)

    ####
    ## create GUI
    ####
    def BuildFrame(self):
        sbar_widths = [-2, -1, -1]
        sbar = self.CreateStatusBar(len(sbar_widths), wx.CAPTION)
        sfont = sbar.GetFont()
        sfont.SetWeight(wx.BOLD)
        sfont.SetPointSize(10)
        sbar.SetFont(sfont)
        self.SetStatusWidths(sbar_widths)

        sizer = wx.BoxSizer(wx.VERTICAL)
        panelkws = self.panelkws
        if self.size is not None:
            panelkws.update({'size': self.size})
        panelkws.update({'output_title': self.output_title,
                         'with_data_process': self.with_data_process,
                         'theme': self.theme})

        self.panel = PlotPanel(self, **panelkws)
        # self.toolbar = NavigationToolbar(self.panel.canvas)
        self.panel.messenger = self.write_message
        self.panel.nstatusbar = sbar.GetFieldsCount()
        sizer.Add(self.panel, 1, wx.EXPAND)
        self.BuildMenu()

        self.SetAutoLayout(True)
        self.SetSizer(sizer)
        self.SetSize(self.GetBestVirtualSize())


    def Build_FileMenu(self, extras=None):
        mfile = wx.Menu()
        MenuItem(self, mfile, "&Save Image\tCtrl+S",
                 "Save Image of Plot (PNG, SVG, JPG)",
                 action=self.save_figure)
        MenuItem(self, mfile, "&Copy\tCtrl+C",
                 "Copy Plot Image to Clipboard",
                 self.Copy_to_Clipboard)
        MenuItem(self, mfile, "Export Data\tCtrl+D",
                 "Export Data to Text File",
                 self.onExport)

        if extras is not None:
            for text, helptext, callback in extras:
                MenuItem(self, mfile, text, helptext, callback)

        mfile.AppendSeparator()
        MenuItem(self, mfile, 'Page Setup...', 'Printer Setup',
                 self.PrintSetup)

        MenuItem(self, mfile, 'Print Preview...', 'Print Preview',
                 self.PrintPreview)

        MenuItem(self, mfile, "&Print\tCtrl+P", "Print Plot",
                 self.Print)

        mfile.AppendSeparator()
        MenuItem(self, mfile, "E&xit\tCtrl+Q", "Exit", self.onExit)
        return mfile

    def Copy_to_Clipboard(self, event=None):
        self.panel.canvas.Copy_to_Clipboard(event=event)

    def PrintSetup(self, event=None):
        self.panel.PrintSetup(event=event)

    def PrintPreview(self, event=None):
        self.panel.PrintPreview(event=event)

    def Print(self, event=None):
        self.panel.Print(event=event)

    def onZoomStyle(self, event=None, style='both x and y'):
        self.panel.conf.zoom_style = style

    def BuildMenu(self):
        mfile = self.Build_FileMenu()
        mopts = wx.Menu()
        MenuItem(self, mopts, "Configure Plot\tCtrl+K",
                 "Configure Plot styles, colors, labels, etc",
                 self.panel.configure)

        MenuItem(self, mopts, "Toggle Legend\tCtrl+L",
                 "Toggle Legend Display",
                 self.panel.toggle_legend)
        MenuItem(self, mopts, "Toggle Grid\tCtrl+G",
                 "Toggle Grid Display",
                 self.panel.toggle_grid)

        mopts.AppendSeparator()

        MenuItem(self, mopts, "Zoom X and Y\tCtrl+W",
                 "Zoom on both X and Y",
                 partial(self.onZoomStyle, style='both x and y'),
                 kind=wx.ITEM_RADIO, checked=True)
        MenuItem(self, mopts, "Zoom X Only\tCtrl+X",
                 "Zoom X only",
                 partial(self.onZoomStyle, style='x only'),
                 kind=wx.ITEM_RADIO)

        MenuItem(self, mopts, "Zoom Y Only\tCtrl+Y",
                 "Zoom Y only",
                 partial(self.onZoomStyle, style='y only'),
                 kind=wx.ITEM_RADIO)

        MenuItem(self, mopts, "Undo Zoom/Pan\tCtrl+Z",
                 "Zoom out / Pan out to previous view",
                 self.panel.unzoom)
        MenuItem(self, mopts, "Zoom all the way out",
                 "Zoom out to full data range",
                 self.panel.unzoom_all)

        mopts.AppendSeparator()

        logmenu = wx.Menu()
        for label in self.panel.conf.log_choices:
            xword, yword = label.split(' / ')
            xscale = xword.replace('x', '').strip()
            yscale = yword.replace('y', '').strip()
            MenuItem(self, logmenu, label, label,
                     partial(self.panel.set_logscale, xscale=xscale, yscale=yscale),
                     kind=wx.ITEM_RADIO)

        mopts.AppendSubMenu(logmenu, "Linear/Log Scale ")

        transmenu = None
        if self.panel.conf.with_data_process:
            transmenu = wx.Menu()
            MenuItem(self, transmenu, "Toggle Derivative", "Toggle Derivative",
                     self.panel.toggle_deriv, kind=wx.ITEM_CHECK)

            for expr in self.panel.conf.data_expressions:
                label = expr
                if label is None:
                    label = 'original Y(X)'
                MenuItem(self, transmenu, label, label,
                         partial(self.panel.process_data, expr=expr),
                         kind=wx.ITEM_RADIO)

        if transmenu is not None:
            mopts.AppendSubMenu(transmenu, "Transform Y(X)")

        mhelp = wx.Menu()
        MenuItem(self, mhelp, "Quick Reference",
                 "Quick Reference for WXMPlot", self.onHelp)
        MenuItem(self, mhelp, "About", "About WXMPlot", self.onAbout)

        mbar = wx.MenuBar()
        mbar.Append(mfile, 'File')
        mbar.Append(mopts, '&Options')
        if self.user_menus is not None:
            for title, menu in self.user_menus:
                mbar.Append(menu, title)

        mbar.Append(mhelp, '&Help')

        self.SetMenuBar(mbar)
        self.Bind(wx.EVT_CLOSE, self.onExit)

    def BindMenuToPanel(self, panel=None):
        pass

    def onAbout(self, event=None):
        dlg = wx.MessageDialog(self, self.about_msg, "About WXMPlot",
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def onExport(self, event=None):
        if (self.panel is None or
            not callable(getattr(self, 'ExportTextFile', None))):
            return

        try:
            title = self.panel.conf.title
        except AttributeError:
            title = None

        if title is None:
            title = self.output_title
        if title is None:
            title = self.GetTitle()

        title = title.strip()
        if title in (None, '', 'None'):
            title = 'wxmplot'

        fname = fix_filename(title + '.dat')

        origdir = get_cwd()
        file_choices = "DAT (*.dat)|*.dat|ALL FILES (*.*)|*.*"
        dlg = wx.FileDialog(self, message='Export Data to Text File',
                            defaultDir=origdir,
                            defaultFile=fname,
                            wildcard=file_choices,
                            style=wx.FD_SAVE|wx.FD_CHANGE_DIR)

        if dlg.ShowModal() == wx.ID_OK:
            self.ExportTextFile(dlg.GetPath(), title=title)
        os.chdir(origdir)

    def onHelp(self, event=None):
        dlg = wx.MessageDialog(self, self.help_msg, "WXMPlot Quick Reference",
                               wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def onExit(self, event=None):
        try:
            if callable(self.exit_callback):
                self.exit_callback()
        except:
            pass
        try:
            if self.panel is not None:
                self.panel.win_config.Close(True)
            if self.panel is not None:
                self.panel.win_config.Destroy()
        except:
            pass

        try:
            self.Destroy()
        except:
            pass




from numpy import exp, random, arange, outer
import wxmplot.interactive as wi

def gauss2d(x, y, x0, y0, sx, sy):
    return outer( exp( -(((y-y0)/float(sy))**2)/2),
                  exp( -(((x-x0)/float(sx))**2)/2) )

ny, nx = 350, 400
x = arange(nx)
y = arange(ny)
ox =  x /200.0
oy = -1 + y / 200.0
dat  = 0.3 * random.random(size=nx*ny).reshape(ny, nx)
dat =  dat + (16.0*gauss2d(x, y, 190,   96,  15,  26) +
              27.0*gauss2d(x, y, 140,  210,  51,  42))

wi.imshow(dat, x=ox, y=oy, style='contour', contour_labels=True)



#!/usr/bin/python

import time
from datetime import datetime
import numpy as np
from wxmplot import PlotApp


npts = 725
y = np.arange(npts) + np.random.normal(scale=3., size=npts)
t0  = time.time() - 700

t = np.array([datetime.fromtimestamp(t0 + i) for i in range(npts)])

app = PlotApp()
app.plot(t, y, use_dates=True,  marker='+')
app.set_title('Time series data:')
app.run()



#!/usr/bin/python
#
# Stacked Plot shows two related plots stacked top and bottom that are
# expected to share an X axis.

# The upper panel is a standard PlotPanel. The bottom panel is compressed
# in height, and follows the X-range when zooming the top panel, and does
# not respond to most other mouse events.

import wx
import numpy
from wxmplot import StackedPlotFrame

x = numpy.arange(0.0, 30.0, 0.1)
noise = numpy.random.normal(size=len(x), scale=0.096)

y1 = numpy.sin(2*x)/(x+2)
y2 = y1 + noise

app = wx.App()

pframe = StackedPlotFrame(title='Stacked Example', ratio=3.000)

pframe.plot(x, y2, label='data(fake)', ylabel='signal', xlabel='x', title='some fit')

pframe.oplot(x, y1, label='simple theory', show_legend=True)
pframe.plot(x, noise, panel='bottom', label='residual', ylabel='residual')


pframe.Show()
pframe.Raise()

app.MainLoop()



#!/usr/bin/python
"""
Plot Configuration of (simplified) properties for Matplotlib Lines.  In matplotlib,
a line is a series of x,y points, which are to be connected into a single trace.

Here, we'll call the thing drawn on the screen a 'trace', and the configuration
here is for trace properties:
    color        color of trace (a color name or hex code such as #FF00FF)
    style        trace style: one of ('solid', 'dashed', 'dotted', 'dash-dot')
    drawstyle    style for joining point: one of ('default', 'steps-pre', 'steps-post')
    width        trace width
    marker       marker symbol for each point (see list below)
    markersize   size of marker
    markercolor  color for marker (defaults to same as trace color
    label        label used for trace


A matplotlib Line2D can have many more properties than this: these are not set here.

Valid marker names are:
   'no symbol', 'o', '+', 'x', '^','v','>','<','|','_',
   'square','diamond','thin diamond', 'hexagon','pentagon',
   'tripod 1','tripod 2'

"""
import time
from copy import copy
import numpy as np
import matplotlib
from matplotlib.font_manager import FontProperties
from matplotlib import rc_params, rcParams
import matplotlib.style
from cycler import cycler
from .colors import hexcolor, mpl_color, mpl2hexcolor

StyleMap  = {}
DrawStyleMap  = {}
MarkerMap = {}

default_config = dict(viewpad=2.5, title='', xscale='linear',
                      yscale='linear', xlabel='', ylabel='', y2label='',
                      plot_type='lineplot', scatter_size=30,
                      scatter_normalcolor='blue',
                      scatter_normaledge='blue', scatter_selectcolor='red',
                      scatter_selectedge='red', auto_margins=True,
                      legend_loc= 'best', legend_onaxis='on plot',
                      draggable_legend=False, hidewith_legend=True,
                      show_legend=False, show_legend_frame=False,
                      axes_style='box', labelfont=9, legendfont=7,
                      titlefont=10, theme='light')


for k in ('default', 'steps-pre','steps-mid', 'steps-post'):
    DrawStyleMap[k] = k

for k,v in (('solid', ('-', None)),
            ('short dashed', ('--',(4, 1))),
            ('dash-dot', ('-.', None)),
            ('dashed', ('--', (6, 6))),
            ('dotted', (':', None)),
            ('long dashed', ('--', (12, 4)))):
    StyleMap[k]=v


for k,v in (('no symbol','None'), ('o','o'), ('+','+'), ('x','x'),
            ('square','s'), ('diamond','D'), ('thin diamond','d'),
            ('^','^'), ('v','v'), ('>','>'), ('<','<'),
            ('|','|'),('_','_'), ('hexagon','h'), ('pentagon','p'),
            ('tripod 1','1'), ('tripod 2','2')):
    MarkerMap[k] = v

ViewPadPercents = [0.0, 2.5, 5.0, 7.5, 10.0]

linecolors = ('#1f77b4', '#d62728', '#2ca02c', '#ff7f0e', '#9467bd',
              '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf')

light_theme = {'axes.grid': True,
               'axes.axisbelow': True,
               'axes.linewidth': 0.5,
               'axes.edgecolor': '#000000',
               'axes.facecolor': '#FEFEFE',
               'grid.linestyle': '-',
               'grid.linewidth': 0.5,
               'lines.linewidth': 2.5,
               'lines.markersize': 3,
               'lines.markeredgewidth': 0.75,
               'xtick.labelsize': 9,
               'ytick.labelsize': 9,
               'legend.fontsize': 8,
               'axes.labelsize': 9,
               'axes.titlesize': 10,
               'xtick.major.size': 4,
               'ytick.major.size': 4,
               'xtick.major.width': 0.5,
               'ytick.major.width': 0.5,
               'xtick.color': '#000000',
               'ytick.color': '#000000',
               'text.color': '#000000',
               'grid.color': '#ECECE5',
               'figure.facecolor': '#FCFCFA',
               'axes.prop_cycle': cycler('color', linecolors),
               'savefig.bbox': None,
               'savefig.directory': '~',
               'savefig.dpi': 'figure',
               'savefig.edgecolor': 'white',
               'savefig.facecolor': 'white',
               'savefig.format': 'png',
               'savefig.orientation': 'portrait',
               'savefig.pad_inches': 0.1,
               }

dark_theme = {'axes.facecolor': '#202020',
              'axes.edgecolor': '#FDFDC0',
              'xtick.color': '#FDFDC0',
              'ytick.color': '#FDFDC0',
              'text.color': '#FDFDC0',
              'grid.color': '#404040',
              'figure.facecolor': '#161616',
              'savefig.edgecolor': 'black',
              'savefig.facecolor': 'black',
              }

whitebg_theme = {'axes.facecolor': '#FFFFFF',
               'figure.facecolor': '#FFFFFF'}


Themes = {}

for tname in ('light', 'white-background', 'dark', 'matplotlib', 'ggplot',
              'bmh', 'fivethirtyeight', 'grayscale', 'dark_background',
              'tableau-colorblind10', 'Solarize_Light2', 'seaborn',
              'seaborn-bright', 'seaborn-colorblind', 'seaborn-dark',
              'seaborn-darkgrid', 'seaborn-dark-palette', 'seaborn-deep',
              'seaborn-notebook', 'seaborn-muted', 'seaborn-pastel',
              'seaborn-paper', 'seaborn-poster', 'seaborn-talk',
              'seaborn-ticks', 'seaborn-white', 'seaborn-whitegrid'):
    theme = rc_params()
    theme['backend'] = 'WXAgg'
    if tname == 'matplotlib':
        pass
    elif tname == 'light':
        theme.update(light_theme)
    elif tname == 'dark':
        theme.update(light_theme)
        theme.update(dark_theme)
    elif tname == 'white-background':
        theme.update(light_theme)
        theme.update(whitebg_theme)
    elif tname == 'fivethirtyeight':  # text sizes are way off the norm
        theme.update(matplotlib.style.library['fivethirtyeight'])
        theme.update({'legend.fontsize': 10, 'xtick.labelsize': 9,
                      'ytick.labelsize': 9, 'axes.labelsize': 9,
                      'axes.titlesize': 13})
    elif tname.startswith('seaborn'):
        sthemes = ['seaborn', 'seaborn-v0_8']
        sname = tname.replace('seaborn', 'seaborn-v0_8')
        for xname in (tname, sname):
            if xname not in sthemes:
                sthemes.append(xname)
        for xname in sthemes:
            if xname in matplotlib.style.library:
                theme.update(matplotlib.style.library[xname])
    elif tname in matplotlib.style.library:
        theme.update(matplotlib.style.library[tname])
    Themes[tname.lower()] = theme

def ifnot_none(val, default):
    "return val if val is not None else default"
    return val if val is not None else default


class LineProps:
    """ abstraction for Line2D properties, closely related to a
    MatPlotlib Line2D.  used to set internal line properties, and
    to  make the matplotlib calls to set the Line2D properties
    """
    REPRFMT = """LineProps(color='{color:s}', style='{style:s}', linewidth={linewidth:.1f},
          label='{label:s}', zorder={zorder:d}, drawstyle='{drawstyle:s}',
          marker='{marker:s}', markersize={markersize:.1f}, markercolor={markercolor:s})"""

    def __init__(self, color='black', style='solid', drawstyle='default',
                 linewidth=2, marker='no symbol',markersize=4,
                 markercolor=None, fill=False, alpha=1.0, zorder=1, label='', mpline=None):
        self.color      = color
        self.alpha      = alpha
        self.style      = style
        self.drawstyle  = drawstyle
        self.fill       = fill
        self.linewidth  = linewidth
        self.marker     = marker
        self.markersize = markersize
        if markercolor is None:
            markercolor = color
        self.markercolor= markercolor
        self.label      = label
        self.zorder     = zorder
        self.mpline     = mpline


    def __repr__(self):
        if self.zorder is None:
            self.zorder = 33
        return self.REPRFMT.format(**self.__dict__)

    def set(self, color=None, style=None, drawstyle=None, linewidth=None,
            marker=None, markersize=None, markercolor=None, zorder=None,
            label=None, fill=False, alpha=None):
        self.color = ifnotNOne(color, self.color)
        self.style = style
        self.drawstyle  = drawstyle
        self.fill       = fill
        self.linewidth  = linewidth
        self.marker     = marker
        self.markersize = markersize
        self.markercolor= markercolor
        self.label      = label
        self.zorder     = zorder
        self.alpha      = alpha


class PlotConfig:
    """Plot Configuration for Line Plots, holding most configuration data """

    def __init__(self, canvas=None, panel=None, with_data_process=True,
                 theme=None, theme_color_callback=None,
                 margin_callback=None, trace_color_callback=None,
                 custom_config=None):

        self.canvas = canvas
        self.panel = panel
        self.styles      = list(StyleMap.keys())
        self.drawstyles  = list(DrawStyleMap.keys())
        self.symbols     = list(MarkerMap.keys())
        self.trace_color_callback = trace_color_callback
        self.theme_color_callback = theme_color_callback
        self.margin_callback = margin_callback
        self.current_theme = theme
        if self.current_theme is None:
            self.current_theme = 'light'
        self.legend_map = {}
        self.legend_locs = ['best', 'upper right' , 'lower right', 'center right',
                            'upper left', 'lower left',  'center left',
                            'upper center', 'lower center', 'center']

        self.legend_abbrevs = {'ur': 'upper right' ,  'ul': 'upper left',
                               'cr': 'center right',  'cl': 'center left',
                               'lr': 'lower right',   'll': 'lower left',
                               'uc': 'upper center',  'lc': 'lower center',
                               'cc': 'center'}

        self.data_expressions = (None, "y*x", "y*x^2", "y^2", "sqrt(y)", "1/y")

        self.log_choices = ("x linear / y linear", "x linear / y log",
                            "x log / y linear", "x log / y log")
        self.zoom_choices = ('both x and y', 'x only', 'y only')
        self.zoom_style = self.zoom_choices[0]
        self.data_deriv = False
        self.data_expr  = None
        self.data_save  = {}
        self.with_data_process = with_data_process

        self.axes_style_choices = ['box', 'open']
        self.legend_onaxis_choices =  ['on plot', 'off plot']

        self.configdict = default_config
        if custom_config is not None:
            self.configdict.update(custom_config)
        self.themes = Themes

        self.set_defaults()

    def set_defaults(self):
        fontsize = {}
        for key, val in self.configdict.items():
            if 'font' in key:
                fontsize[key] = val
            else:
                setattr(self, key, val)

        self.zoom_lims = []
        self.added_texts = []
        self.scatter_xdata = None
        self.scatter_ydata = None
        self.scatter_mask = None
        self.margins = None
        self.mpl_legend  = None
        self.axes_traces = {}

        # preload some traces
        self.traces = []
        self.reset_lines()

        f0 =  FontProperties()
        self.labelfont = f0.copy()
        self.titlefont = f0.copy()
        self.legendfont = f0.copy()
        self.legendfont.set_size(fontsize['legendfont'])
        self.labelfont.set_size(fontsize['labelfont'])
        self.titlefont.set_size(fontsize['titlefont'])
        self.set_theme()

    def set_theme(self, theme=None):
        if theme in self.themes:
            self.current_theme = theme

        cur_theme = self.themes[self.current_theme]
        rcParams.update(cur_theme)

        self.show_grid  = cur_theme['axes.grid']
        self.legendfont.set_size(cur_theme['legend.fontsize'])
        self.labelfont.set_size(cur_theme['axes.labelsize'])
        self.titlefont.set_size(cur_theme['axes.titlesize'])

        self.set_facecolor(mpl2hexcolor(cur_theme['axes.facecolor']))
        self.set_gridcolor(mpl2hexcolor(cur_theme['grid.color']))
        self.set_framecolor(mpl2hexcolor(cur_theme['figure.facecolor']))
        self.set_textcolor(mpl2hexcolor(cur_theme['text.color']))

        self.linecolors = [a['color'] for a in cur_theme['axes.prop_cycle']]
        self.reset_trace_properties()
        self.set_axes_style()

    def get_current_config(self):
        """save dict of current configuration options to self.configdict
        """
        cnf = {}
        for key in self.configdict:
            if hasattr(self, key):
                val = getattr(self, key)
                if key in ('legendfont', 'labelfont', 'titlefont'):
                    val = val.get_size()
                elif key == 'linecolors':
                    val = [trace.color for trace in self.traces[:10]]
                cnf[key] = val
        self.configdict = cnf
        return cnf

    def load_config(self, conf):
        self.get_current_config()
        self.configdict.update(conf)
        self.set_defaults()

    def reset_lines(self):
        self.lines = [None]*len(self.traces)
        self.dy    = [None]*len(self.traces)
        self.fills = [None]*len(self.traces)
        self.ntrace = 0
        return self.lines

    def reset_trace_properties(self):
        i = -1
        t0 = time.time()
        for style, marker in (('solid', None), ('short dashed', None),
                              ('dash-dot', None), ('solid', 'o'),
                              ('dotted', None), ('solid', '+'),
                              ('dashed', None), ('solid', 'x'),
                              ('long dashed', None), ('dashed', 'square')):
            for color in self.linecolors:
                i += 1
                self.init_trace(i, color, style, marker=marker)

    def init_trace(self, n, color, style, label=None, linewidth=None,
                   zorder=None, marker=None, markersize=None,
                   drawstyle=None, fill=None, alpha=1):
        """ used for building set of traces"""
        while n >= len(self.traces):
            self.traces.append(LineProps())
            self.fills.append(None)
            self.dy.append(None)
        line = self.traces[n]

        line.label     = ifnot_none(label, "trace %i" % (n+1))
        line.color     = ifnot_none(color, line.color)
        line.alpha     = ifnot_none(alpha, line.alpha)
        line.style     = ifnot_none(style, line.style)
        line.linewidth = ifnot_none(linewidth, line.linewidth)
        line.drawstyle = ifnot_none(drawstyle, line.drawstyle)
        line.fill      = ifnot_none(fill, line.fill)
        line.zorder    = ifnot_none(zorder, 5*(n+1))
        line.marker    = ifnot_none(marker, line.marker)
        line.markersize = ifnot_none(markersize, line.markersize)


    def get_mpline(self, trace):
        n = max(0, int(trace))
        while n >= len(self.traces):
            self.traces.append(LineProps())
        nlines = len(self.lines)
        for i in range(nlines, len(self.traces)+1):
            self.lines.append(None)
        try:
            return self.lines[n]
        except:
            return self.lines[n-1]


    def get_trace(self, trace):
        if trace is None:
            trace = self.ntrace
        while trace >= len(self.traces):
            self.traces.append(LineProps())
        return trace

    def relabel(self, xlabel=None, ylabel=None,
                y2label=None, title=None, delay_draw=False):
        " re draw labels (title, x, y labels)"
        n = self.labelfont.get_size()
        # self.titlefont.set_size(n+1)
        rcParams['xtick.labelsize'] =  rcParams['ytick.labelsize'] =  n

        if xlabel is not None:
            self.xlabel = xlabel
        if ylabel is not None:
            self.ylabel = ylabel
        if y2label is not None:
            self.y2label = y2label
        if title is not None:
            self.title = title
        if self.canvas is None: return
        axes = self.canvas.figure.get_axes()
        kws = dict(fontproperties=self.titlefont, color=self.textcolor)
        axes[0].set_title(self.title, **kws)
        kws['fontproperties'] = self.labelfont

        if len(self.xlabel) > 0 and self.xlabel not in ('', None, 'None'):
            axes[0].set_xlabel(self.xlabel, **kws)

        if len(self.ylabel) > 0 and self.ylabel not in ('', None, 'None'):
            axes[0].set_ylabel(self.ylabel, **kws)
        if (len(axes) > 1 and len(self.y2label) > 0 and
            self.y2label not in ('', None, 'None')):
            axes[1].set_ylabel(self.y2label, **kws)

        for ax in axes[0].xaxis, axes[0].yaxis:
            for t in ax.get_ticklabels():
                t.set_color(self.textcolor)
                if hasattr(t, 'set_fontsize'):
                    t.set_fontsize(n)

        self.set_added_text_size()
        if self.mpl_legend is not None:
            for t in self.mpl_legend.get_texts():
                t.set_color(self.textcolor)
        if not delay_draw:
            self.canvas.draw()

    def set_margins(self, left=0.1, top=0.1, right=0.1, bottom=0.1,
                    delay_draw=False):
        "set margins"
        self.margins = (left, top, right, bottom)
        if self.panel is not None:
            self.panel.gridspec.update(left=left, top=1-top,
                                       right=1-right, bottom=bottom)
        for ax in self.canvas.figure.get_axes():
            # ax.update_params()
            figpos = ax.get_subplotspec().get_position(self.canvas.figure)
            ax.set_position(figpos)

        if not delay_draw:
            self.canvas.draw()
        if callable(self.margin_callback):
            self.margin_callback(left=left, top=top, right=right, bottom=bottom)

    def set_gridcolor(self, color):
        """set color for grid"""
        self.gridcolor = color
        if self.canvas is None: return

        rcParams['grid.color'] = color
        for ax in self.canvas.figure.get_axes():
            for i in ax.get_xgridlines()+ax.get_ygridlines():
                i.set_color(color)
                i.set_zorder(-1)
        if callable(self.theme_color_callback):
            self.theme_color_callback(color, 'grid')

    def set_facecolor(self, color):
        """set color for background of plot"""
        self.facecolor = color
        if self.canvas is None: return
        for ax in self.canvas.figure.get_axes():
            ax.set_facecolor(color)
        if callable(self.theme_color_callback):
            self.theme_color_callback(color, 'axes.facecolor')

    def set_framecolor(self, color):
        """set color for outer frame"""
        self.framecolor = color
        if self.canvas is None: return
        self.canvas.figure.set_facecolor(color)
        if callable(self.theme_color_callback):
            self.theme_color_callback(color, 'figure.facecolor')

    def set_textcolor(self, color, delay_draw=True):
        """set color for labels and axis text"""
        self.textcolor = color
        self.relabel(delay_draw=delay_draw)
        if callable(self.theme_color_callback):
            self.theme_color_callback(color, 'text')


    def set_added_text_size(self):
        # for text added to plot, reset font size to match legend
        n = self.legendfont.get_size()
        for dynamic, txt in self.added_texts:
            if dynamic:
                txt.set_fontsize(n)

    def refresh_trace(self, trace=None):
        trace = self.get_trace(trace)
        prop = self.traces[trace]

        self.set_trace_label(prop.label, trace=trace, delay_draw=True)
        self.set_trace_linewidth(prop.linewidth, trace=trace, delay_draw=True)
        self.set_trace_color(prop.color, trace=trace, delay_draw=True)
        self.set_trace_alpha(prop.alpha, trace=trace, delay_draw=True)
        self.set_trace_style(prop.style, trace=trace, delay_draw=True)
        self.set_trace_drawstyle(prop.drawstyle, trace=trace, delay_draw=True)
        self.set_trace_fill(prop.fill, trace=trace, delay_draw=True)
        self.set_trace_marker(prop.marker, trace=trace, delay_draw=True)
        self.set_trace_markersize(prop.markersize, trace=trace, delay_draw=True)
        self.set_trace_zorder(prop.zorder, trace=trace, delay_draw=True)

    def set_trace_color(self, color, trace=None, delay_draw=True):
        trace = self.get_trace(trace)
        color = hexcolor(color)
        self.traces[trace].color = color
        mline = self.get_mpline(trace)
        if mline:
            for comp in mline:
                if hasattr(comp, '__iter__'):
                    for l in comp:
                        l.set_color(color)
                else:
                    comp.set_color(color)

        if self.fills[trace] is not None:
            self.fills[trace].set_color(color)

        if not delay_draw:
            self.draw_legend()
        if callable(self.trace_color_callback) and mline:
            self.trace_color_callback(color, line=mline)

    def set_trace_alpha(self, alpha, trace=None, delay_draw=True):
        trace = self.get_trace(trace)
        alpha = min(1, max(0, float(alpha)))
        self.traces[trace].alpha = alpha
        mline = self.get_mpline(trace)
        if mline:
            for comp in mline:
                if hasattr(comp, '__iter__'):
                    for l in comp:
                        l.set_alpha(alpha)
                else:
                    comp.set_alpha(alpha)
        if self.fills[trace] is not None:
            self.fills[trace].set_alpha(alpha)
        if not delay_draw:
            self.draw_legend()


    def set_trace_zorder(self, zorder, trace=None, delay_draw=False):
        trace = self.get_trace(trace)
        zorder = ifnot_none(zorder, 5*(trace+1))
        self.traces[trace].zorder = zorder
        mline = self.get_mpline(trace)
        if mline:
            mline[0].set_zorder(zorder)

        if not delay_draw:
            self.canvas.draw()

    def set_trace_label(self, label, trace=None, delay_draw=False):
        trace = self.get_trace(trace)
        self.traces[trace].label = label
        mline = self.get_mpline(trace)
        if mline:
            mline[0].set_label(label)
        if not delay_draw:
            self.draw_legend()

    def set_trace_style(self, style, trace=None, delay_draw=False):
        trace = self.get_trace(trace)
        sty = 'solid'
        if style in StyleMap:
            sty = style
        elif style in StyleMap.values():
            for k,v in StyleMap.items():
                if v == style:  sty = k
        style = sty
        self.traces[trace].style = style
        mline = self.get_mpline(trace)
        if mline:
            _key, _opts = StyleMap[style]
            mline[0].set_linestyle(_key)
            if _key == '--' and _opts is not None:
                mline[0].set_dashes(_opts)
        if not delay_draw:
            self.draw_legend()

    def set_trace_drawstyle(self, drawstyle, trace=None, delay_draw=False):
        trace = self.get_trace(trace)
        sty = 'default'
        if drawstyle in DrawStyleMap:
            sty = drawstyle
        drawstyle = sty
        self.traces[trace].drawstyle = drawstyle

        mline = self.get_mpline(trace)
        if mline:
            mline[0].set_drawstyle(DrawStyleMap[drawstyle])
            mline[0]._invalidx = True

        if not delay_draw:
            self.draw_legend()

    def set_trace_fill(self, fill, trace=None, delay_draw=False):
        trace = self.get_trace(trace)

        cur_fill = self.fills[trace]
        axes = self.canvas.figure.get_axes()[0]

        def del_collection(thisfill):
            for i, coll in enumerate(axes.collections):
                if id(thisfill) == id(coll):
                    try:
                        axes.collections[i].remove()
                    except:
                        pass

        if not fill:
            self.traces[trace].fill = False
            if cur_fill is not None:
                del_collection(cur_fill)
                try:
                    del cur_fill
                except:
                    pass
                self.fills[trace] = None
        else:
            if cur_fill is not None:
                del_collection(cur_fill)
                try:
                    del cur_fill
                except:
                    pass


            self.traces[trace].fill = True
            atrace = self.traces[trace]
            this = self.get_mpline(trace)
            dy  = self.dy[trace]
            if this is not None:
                fkws = dict(step=None, zorder=atrace.zorder,
                            color=atrace.color,
                            alpha=atrace.alpha)
                if atrace.drawstyle != 'default':
                    fkws['step'] = drawstyle
                x = this[0].get_xdata()
                y = this[0].get_ydata()
                y2 = 0
                if dy is not None:
                    y, y2 = y-dy, y+dy

                _fill = axes.fill_between(x, y, y2=y2, **fkws)

                self.fills[trace] = _fill
        if not delay_draw:
            self.draw_legend()

    def set_trace_marker(self, marker, trace=None, delay_draw=False):
        trace = self.get_trace(trace)
        sym = 'no symbol'
        if marker in MarkerMap:
            sym = marker
        elif marker in MarkerMap.values():
            for k,v in MarkerMap.items():
                if v == marker:  sym = k
        marker = sym
        self.traces[trace].marker = marker

        mline = self.get_mpline(trace)
        if mline:
            mline[0].set_marker(MarkerMap[marker])

        if not delay_draw:
            self.draw_legend()

    def set_trace_markersize(self, markersize, trace=None, delay_draw=False):
        trace = self.get_trace(trace)
        self.traces[trace].markersize = markersize

        mline = self.get_mpline(trace)
        if mline:
            mline[0].set_markersize(markersize)
            if mline[0].get_markeredgewidth() == 0:
                mline[0].set_markeredgewidth(0.75)

        if not delay_draw:
            self.draw_legend()

    def set_trace_linewidth(self, linewidth, trace=None, delay_draw=False):
        trace = self.get_trace(trace)
        self.traces[trace].linewidth = linewidth

        mline = self.get_mpline(trace)
        if mline:
            for line in mline:
                try:
                    line.set_linewidth(linewidth/2.0)
                except:
                    pass

        if not delay_draw:
            self.draw_legend()

    def set_trace_datarange(self, datarange, trace=None):
        pass


    def get_mpl_line(self, trace=None):
        this = self.get_mpline(self.get_trace(trace))
        if this is None:
            trace = 5
            while this is None and trace > 0:
                trace = trace - 1
                this = self.get_mpline(self.get_trace(trace))
        return this[0]


    def enable_grid(self, show=None, delay_draw=False):
        "enable/disable grid display"
        if show is not None:
            self.show_grid = show
        axes = self.canvas.figure.get_axes()
        ax0 = axes[0]
        for i in ax0.get_xgridlines() + ax0.get_ygridlines():
            i.set_color(self.gridcolor)
            i.set_zorder(-1)
        axes[0].grid(self.show_grid)
        for ax in axes[1:]:
            ax.grid(False)
        if not delay_draw:
            self.canvas.draw()

    def set_axes_style(self, style=None, delay_draw=False):
        """set axes style: one of
           'box' / 'fullbox'  : show all four axes borders
           'open' / 'leftbot' : show left and bottom axes
           'bottom'           : show bottom axes only
        """
        if style is not None:
            self.axes_style = style

        try:
            axes = self.canvas.figure.get_axes()
            ax = axes[0]
        except AttributeError:
            return



        for tline in (ax.xaxis.get_majorticklines() + ax.yaxis.get_majorticklines()):
            tline.set_color(rcParams['xtick.color'])
            tline.set_markeredgewidth(rcParams['xtick.major.width'])
            tline.set_markersize(rcParams['xtick.major.size'])
            tline.set_linestyle('-')
            tline.set_visible(True)

        col = rcParams['axes.edgecolor']
        for spine in ('top', 'bottom', 'left', 'right'):
            ax.spines[spine].set_linewidth(rcParams['axes.linewidth'])
            ax.spines[spine].set_facecolor(rcParams['axes.facecolor'])
            ax.spines[spine].set_edgecolor(rcParams['axes.edgecolor'])

        _sty = self.axes_style.lower()
        if  _sty in ('fullbox', 'full'):
            _sty = 'box'
        if  _sty == 'leftbot':
            _sty = 'open'

        if _sty == 'box':
            ax.xaxis.set_ticks_position('both')
            ax.yaxis.set_ticks_position('both')
            ax.spines['top'].set_visible(True)
            ax.spines['right'].set_visible(True)
        elif _sty == 'open':
            ax.xaxis.set_ticks_position('bottom')
            ax.yaxis.set_ticks_position('left')
            ax.spines['top'].set_visible(False)
            ax.spines['right'].set_visible(False)
        elif _sty == 'bottom':
            ax.xaxis.set_ticks_position('bottom')
            ax.spines['top'].set_visible(False)
            ax.spines['left'].set_visible(False)
            ax.spines['right'].set_visible(False)

        # if 2 y-axes, don't draw ticks on right side for axes[0]
        if len(axes) > 1:
            axes[0].yaxis.set_tick_params(right=False)
            axes[1].yaxis.set_tick_params(left=False)

        if not delay_draw:
            self.canvas.draw()

    def draw_legend(self, show=None, auto_location=True, delay_draw=False):
        "redraw the legend"
        if show is not None:
            self.show_legend = show
        axes = self.canvas.figure.get_axes()
        # clear existing legend
        try:
            lgn = self.mpl_legend
            if lgn:
                for i in lgn.get_texts():
                    i.set_text('')
                for i in lgn.get_lines():
                    i.set_linewidth(0)
                    i.set_markersize(0)
                    i.set_marker('None')
                lgn.draw_frame(False)
                lgn.set_visible(False)
        except:
            pass

        labs = []
        lins = []
        traces = []
        for ax in axes:
            for trace, xline in enumerate(ax.get_lines()):
                xlab = xline.get_label()
                if (xlab != '_nolegend_' and len(xlab)>0):
                    lins.append(xline)
                    traces.append(trace)

        for l in lins:
            xl = l.get_label()
            if not self.show_legend: xl = ''
            labs.append(xl)
        labs = tuple(labs)

        lgn = axes[0].legend
        if self.legend_onaxis.startswith('off'):
            lgn = self.canvas.figure.legend
            # 'best' and 'off axis' not implemented yet
            if self.legend_loc == 'best':
                self.legend_loc = 'upper right'

        if self.show_legend:
            self.mpl_legend = lgn(lins, labs, prop=self.legendfont,
                                  loc=self.legend_loc)
            self.mpl_legend.draw_frame(self.show_legend_frame)
            facecol = axes[0].get_facecolor()

            self.mpl_legend.legendPatch.set_facecolor(facecol)
            if self.draggable_legend:
                self.mpl_legend.set_draggable(True, update='loc')
            self.legend_map = {}
            for legline, legtext, mainline, trace in zip(self.mpl_legend.get_lines(),
                                                         self.mpl_legend.get_texts(),
                                                         lins, traces):
                legline.set_pickradius(20)
                legtext.set_picker(5)
                self.legend_map[legline] = (mainline, trace, legline, legtext)
                self.legend_map[legtext] = (mainline, trace, legline, legtext)
                legtext.set_color(self.textcolor)

        self.set_added_text_size()
        if not delay_draw:
            self.canvas.draw()

    def set_legend_location(self, loc, onaxis):
        "set legend location"
        self.legend_onaxis = 'on plot'
        if not onaxis:
            self.legend_onaxis = 'off plot'
            if loc == 'best':
                loc = 'upper right'
        if loc in self.legend_abbrevs:
            loc = self.legend_abbrevs[loc]
        if loc in self.legend_locs:
            self.legend_loc = loc

    def process_data(self):
        if not self.with_data_process:
            return
        expr = self.data_expr
        if expr is not None:
            expr = expr.upper()
        for ax in self.canvas.figure.get_axes():
            for trace, lines in enumerate(ax.get_lines()):
                try:
                    dats = copy(self.data_save[ax][trace])
                except:
                    return
                xd, yd = np.asarray(dats[0][:]), np.asarray(dats[1][:])
                if expr == 'Y*X':
                    yd = yd * xd
                elif expr == 'Y*X^2':
                    yd = yd * xd * xd
                elif expr == 'Y^2':
                    yd = yd * yd
                elif expr == 'SQRT(Y)':
                    yd = np.sqrt(yd)
                elif expr == '1/Y':
                    yd = 1./yd
                if self.data_deriv:
                    yd = np.gradient(yd)/np.gradient(xd)
                lines.set_ydata(yd)
                lines.set_xdata(xd)

        self.unzoom(full=True)

    def unzoom(self, full=False, delay_draw=False):
        """unzoom display 1 level or all the way"""
        if full:
            self.zoom_lims = self.zoom_lims[:1]
            self.zoom_lims = []
        elif len(self.zoom_lims) > 0:
            self.zoom_lims.pop()
        self.set_viewlimits()
        if not delay_draw:
            self.canvas.draw()

    def set_viewlimits(self):
        all_limits = []
        x_minpos = None
        y_minpos = None
        for ax in self.canvas.figure.get_axes():
            limits = [None, None, None, None]
            if ax in self.axes_traces:
                try:
                    for trace, lines in enumerate(ax.get_lines()):
                        if lines.get_label() == '_nolegend_':
                            continue
                        x, y = lines.get_xdata(), lines.get_ydata()
                        try:
                            if not isinstance(y, np.ndarray):
                                y = np.array(y)
                            if not isinstance(x, np.ndarray):
                                x = np.array(x)
                            if y_minpos is None:
                                y_minpos= min(y[np.where(y>0)])
                            else:
                                y_minpos= min(y_minpos, min(y[np.where(y>0)]))
                            if x_minpos is None:
                                x_minpos= min(x[np.where(x>0)])
                            else:
                                x_minpos= min(x_minpos, min(x[np.where(x>0)]))
                        except:
                            pass
                        if limits == [None, None, None, None]:
                            limits = [min(x), max(x), min(y), max(y)]
                        else:
                            limits = [min(limits[0], min(x)),
                                      max(limits[1], max(x)),
                                      min(limits[2], min(y)),
                                      max(limits[3], max(y))]
                except ValueError:
                    pass
            if x_minpos is None: x_minpos = 1.e-8
            if y_minpos is None: y_minpos = 1.e-8
            if ax in self.user_limits:
                for i, val in  enumerate(self.user_limits[ax]):
                    if val is not None:
                        limits[i] = val
            # add padding to data range
            if self.viewpad > 0 and (None not in limits):
                xrange = limits[1] - limits[0]
                try:
                    if xrange < 1.e-10:
                        xrange = max(1.e-10, (limits[1] + limits[0] )/2.0)
                except:
                    pass

                yrange = limits[3] - limits[2]
                try:
                    if yrange < 1.e-10:
                        yrange = max(1.e-10, (limits[3] + limits[2] )/2.0)
                except:
                    pass

                limits[0] = limits[0] - xrange * self.viewpad /100.0
                limits[1] = limits[1] + xrange * self.viewpad /100.0
                limits[2] = limits[2] - yrange * self.viewpad /100.0
                limits[3] = limits[3] + yrange * self.viewpad /100.0
                if self.xscale == 'log':
                    limits[0] = max(x_minpos/2, limits[0])

                if self.yscale == 'log':
                    limits[2] = max(y_minpos/2, limits[2])


            if ax in self.user_limits:
                for i, val in  enumerate(self.user_limits[ax]):
                    if val is not None:
                        limits[i] = val

            if len(self.zoom_lims) > 0:
                limits = self.zoom_lims[-1][ax]
            all_limits.append(limits)
            try:
                ax.set_xlim((limits[0], limits[1]), emit=True)
            except:
                pass
            try:
                ax.set_ylim((limits[2], limits[3]), emit=True)
            except:
                pass
        return all_limits

    def set_logscale(self, xscale='linear', yscale='linear',
                     delay_draw=False):
        "set log or linear scale for x, y axis"
        self.xscale = xscale
        self.yscale = yscale
        for axes in self.canvas.figure.get_axes():
            try:
                axes.set_yscale(yscale)
            except:
                axes.set_yscale('linear')
            try:
                axes.set_xscale(xscale)
            except:
                axes.set_xscale('linear')
        if not delay_draw:
            self.process_data()

    def get_viewpads(self):
        o = [round(i*100.0) for i in ViewPadPercents]
        cur = round(100.0*self.viewpad)
        if cur not in o:
            o.append(cur)
        return [i/100.0 for i in sorted(o)]



#
# read Pilatus image into numpy array
import sys
import wx
from tifffile import imread
from wxmplot import ImageFrame

dat = imread('Pilatus.tiff')

app = wx.App()
frame = ImageFrame()
frame.display(dat)
frame.Show()
app.MainLoop()



#!/usr/bin/env python
# simple wxmplot import

import numpy
import matplotlib

def test_import():
    success = False
    try:
        import wxmplot
        success = True
    except:
        pass
    assert(success)


def test_interactive_import():
    success = False
    try:
        import wxmplot.interactive as wi
        success = True
    except:
        pass
    assert(success)



import sys
import time, os, sys

from numpy import arange, sin, cos, exp, pi, linspace, ones, random

import wx
from wxmplot.plotframe import PlotFrame

class TestFrame(wx.Frame):
    def __init__(self, parent=None, *args,**kwds):

        kwds["style"] = wx.DEFAULT_FRAME_STYLE|wx.RESIZE_BORDER|wx.TAB_TRAVERSAL

        wx.Frame.__init__(self, parent, -1, '',
                         wx.DefaultPosition, wx.Size(-1,-1), **kwds)
        self.SetTitle(" WXMPlot Plotting Demo")

        self.SetFont(wx.Font(12,wx.SWISS,wx.NORMAL,wx.BOLD,False))
        menu = wx.Menu()
        menu_exit = menu.Append(-1, "E&xit", "Terminate the program")

        menuBar = wx.MenuBar()
        menuBar.Append(menu, "&File");
        self.SetMenuBar(menuBar)

        self.Bind(wx.EVT_MENU, self.OnExit, menu_exit)

        self.Bind(wx.EVT_CLOSE, self.OnExit)

        self.plotframe  = None

        self.create_data()
        framesizer = wx.BoxSizer(wx.VERTICAL)

        panel      = wx.Panel(self, -1, size=(-1, -1))
        panelsizer = wx.BoxSizer(wx.VERTICAL)

        panelsizer.Add( wx.StaticText(panel, -1, 'wxmplot PlotPanel examples '),
                        0, wx.ALIGN_LEFT|wx.LEFT|wx.EXPAND, 10)

        b10 = wx.Button(panel, -1, 'Example #1',          size=(-1,-1))
        b20 = wx.Button(panel, -1, 'Example #2',          size=(-1,-1))
        b22 = wx.Button(panel, -1, 'Plot with 2 axes',    size=(-1,-1))
        b31 = wx.Button(panel, -1, 'Plot with Errorbars', size=(-1,-1))
        b32 = wx.Button(panel, -1, 'SemiLog Plot',        size=(-1,-1))
        b40 = wx.Button(panel, -1, 'Start Timed Plot',    size=(-1,-1))
        b50 = wx.Button(panel, -1, 'Stop Timed Plot',     size=(-1,-1))
        b60 = wx.Button(panel, -1, 'Plot 500,000 points',  size=(-1,-1))
        bmany1 = wx.Button(panel, -1, 'Plot 20 traces (delay_draw=False)', size=(-1,-1))
        bmany2 = wx.Button(panel, -1, 'Plot 20 traces (delay_draw=True)', size=(-1,-1))
        bmany3 = wx.Button(panel, -1, 'Plot 20 traces (use plot_many())', size=(-1,-1))

        b10.Bind(wx.EVT_BUTTON,self.onPlot1)
        b20.Bind(wx.EVT_BUTTON,self.onPlot2)
        b22.Bind(wx.EVT_BUTTON,self.onPlot2Axes)
        b31.Bind(wx.EVT_BUTTON,self.onPlotErr)
        b32.Bind(wx.EVT_BUTTON,self.onPlotSLog)
        b40.Bind(wx.EVT_BUTTON,self.onStartTimer)
        b50.Bind(wx.EVT_BUTTON,self.onStopTimer)
        b60.Bind(wx.EVT_BUTTON,self.onPlotBig)

        bmany1.Bind(wx.EVT_BUTTON,self.onPlotMany_Slow)
        bmany2.Bind(wx.EVT_BUTTON,self.onPlotMany_Delay)
        bmany3.Bind(wx.EVT_BUTTON,self.onPlotMany_Fast)

        panelsizer.Add(b10, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(b20, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(b22, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(b31, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(b32, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(b40, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(b50, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(b60, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)

        panelsizer.Add(bmany1, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(bmany2, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)
        panelsizer.Add(bmany3, 0, wx.ALIGN_LEFT|wx.ALIGN_CENTER|wx.LEFT, 5)

        panel.SetSizer(panelsizer)
        panelsizer.Fit(panel)

        framesizer.Add(panel, 0, wx.ALIGN_LEFT|wx.EXPAND,2)
        self.SetSizer(framesizer)
        framesizer.Fit(self)

        self.Bind(wx.EVT_TIMER, self.onTimer)
        self.timer = wx.Timer(self)
        self.Refresh()

    def create_data(self):
        self.count = 0
        self.x  = x = arange(0.0,25.0,0.1)
        self.y1 = 4*cos(2*pi*(x-1)/5.7)/(6+x) + 2*sin(2*pi*(x-1)/2.2)/(10)
        self.y2 = sin(2*pi*x/30.0)
        self.y3 =  -pi + 2*(x/10. + exp(-(x-3)/5.0))
        self.y4 =  exp(0.01 + 0.5*x ) / (x+2)
        self.y5 =  3000 * self.y3
        self.npts = len(self.x)
        self.bigx   = linspace(0, 2500, 500000)
        self.bigy   = (sin(pi*self.bigx/140.0) +
                       cos(pi*self.bigx/277.0) +
                       cos(pi*self.bigx/820.0))


        self.many_dlist = [(self.x, self.y1)]
        for i in range(19):
            self.many_dlist.append((self.x, sin(2*(i+1)*x/23.0)))


    def ShowPlotFrame(self, do_raise=True, clear=True):
        "make sure plot frame is enabled, and visible"
        if self.plotframe is None:
            self.plotframe = PlotFrame(self)
            self.has_plot = False
        try:
            self.plotframe.Show()
        except RuntimeError:
            self.plotframe = PlotFrame(self)
            self.plotframe.Show()

        if do_raise:
            self.plotframe.Raise()
        if clear:
            self.plotframe.panel.clear()
            self.plotframe.reset_config()


    def onPlot1(self,event=None):
        self.ShowPlotFrame()
        self.plotframe.plot(self.x,self.y1)
        self.plotframe.oplot(self.x,self.y2)
        self.plotframe.write_message("Plot 1")

    def onPlot2(self,event=None):
        self.ShowPlotFrame()
        x  = arange(100)
        y1 = cos(pi*x/72)
        y2 = sin(pi*x/23)
        self.plotframe.plot(x, y1, color='red')
        self.plotframe.oplot(x, y2, color='green3', marker='+')
        self.plotframe.write_message("Plot 2")


    def onPlotErr(self,event=None):
        self.ShowPlotFrame()
        npts = 81
        x  = linspace(0, 40.0, npts)
        y  = 0.4 * cos(x/2.0) + random.normal(scale=0.03, size=npts)
        dy = 0.03 * (ones(npts) + random.normal(scale=0.2, size=npts))

        self.plotframe.plot(x, y, dy=dy, color='red', linewidth=0,
                            xlabel='x', ylabel='y', marker='o',
                            title='Plot with error bars')
        self.plotframe.write_message("Errorbars!")


    def onPlot2Axes(self,event=None):
        self.ShowPlotFrame()

        self.plotframe.plot( self.x,self.y2, color='black',style='dashed')
        self.plotframe.oplot(self.x,self.y5, color='red', side='right')
        self.plotframe.write_message("Plot with 2 axes")


    def onPlotSLog(self,event=None):
        self.ShowPlotFrame()

        self.plotframe.plot( self.x,self.y4,ylog_scale=True,
                             color='black',style='dashed')
        self.plotframe.write_message("Semi Log Plot")

    def onPlotBig(self,event=None):
        self.ShowPlotFrame()

        t0 = time.time()
        self.plotframe.plot(self.bigx, self.bigy, marker='+', linewidth=0)
        dt = time.time()-t0
        self.plotframe.write_message(
            "Plot array with npts=%i, elapsed time=%8.3f s" % (len(self.bigx),dt))

    def onPlotMany_Slow(self, event=None):
        self.ShowPlotFrame()
        dlist = self.many_dlist

        t0 = time.time()
        opts = dict(title='Plot 20 traces without delay_draw',
            show_legend=True, xlabel='x')

        self.plotframe.plot(dlist[0][0], dlist[0][1], **opts)
        for tdat in dlist[1:]:
            self.plotframe.oplot(tdat[0], tdat[1])

        dt = time.time()-t0
        self.plotframe.write_message(
            "Plot 20 traces without delay_draw, time=%8.3f s" % (dt))

    def onPlotMany_Delay(self, event=None):
        self.ShowPlotFrame()
        dlist = self.many_dlist

        t0 = time.time()
        opts0 = dict(title='Plot 20 traces with delay_draw',
                     show_legend=True, xlabel='x', new=True, delay_draw=True)
        opts1 = dict(new=False, delay_draw=True)
        for i, tdat in enumerate(dlist):
            opts = opts0 if i == 0 else opts1
            if i == len(dlist)-1: opts['delay_draw'] = False
            self.plotframe.oplot(tdat[0], tdat[1], **opts)

        dt = time.time()-t0
        self.plotframe.write_message(
            "Plot 20 traces with delay_draw, time=%8.3f s" % (dt))

    def onPlotMany_Fast(self, event=None):
        self.ShowPlotFrame()
        dlist = self.many_dlist

        t0 = time.time()
        opts = dict(title='Plot 20 traces using plot_many()',
            show_legend=True, xlabel='x')
        self.plotframe.plot_many(dlist, **opts)

        dt = time.time()-t0
        self.plotframe.write_message(
            "Plot 20 traces with plot_many(), time=%8.3f s" % (dt))

    def report_memory(i):
        pid = os.getpid()
        if os.name == 'posix':
            mem = os.popen("ps -o rss -p %i" % pid).readlines()[1].split()[0]
        else:
            mem = 0
        return int(mem)

    def onStartTimer(self,event=None):
        self.count    = 0
        self.up_count = 0
        self.n_update = 1
        self.datrange = None
        self.time0    = time.time()
        self.start_mem= self.report_memory()
        self.timer.Start(10)

    def onStopTimer(self,event=None):
        self.timer.Stop()

    def onTimer(self, event):
        # print 'timer ', self.count, time.time()
        self.count += 1
        n = self.count
        if n < 2:
            self.ShowPlotFrame(do_raise=False, clear=False)
            return
        if n <= 3:
            self.plotframe.plot(self.x[:n], self.y1[:n])# , grid=False)

        else:
            self.plotframe.update_line(0, self.x[:n], self.y1[:n], update_limits=True, draw=True)
            etime = time.time() - self.time0
            rate  = n / max(0.004, etime)
            s = " %i / %i points in %8.2f sec: %8.2f draws/sec" % (n,self.npts,etime, rate)

            self.plotframe.write_message(s)
        if n >= self.npts:
            self.timer.Stop()


    def OnAbout(self, event):
        dlg = wx.MessageDialog(self, "This sample program shows some\n"
                              "examples of WXMPlot PlotFrame.\n"
                              "message dialog.",
                              "About WXMPlot test", wx.OK | wx.ICON_INFORMATION)
        dlg.ShowModal()
        dlg.Destroy()

    def OnExit(self, event):
        try:
            if self.plotframe != None:  self.plotframe.onExit()
        except:
            pass
        self.Destroy()

if __name__ == '__main__':
    app = wx.App()
    f = TestFrame(None,-1)
    f.Show(True)
    app.MainLoop()



#!/usr/bin/python
##
## ResidualPlotFrame: a wx.Frame for 2 PlotPanels, top and bottom
##   with the top panel being the main panel and the lower panel
##   being 1/4 the height (configurable) and the dependent panel

from .stackedplotframe import StackedPlotFrame

class ResidualPlotFrame(StackedPlotFrame):
    """
    Top/Bottom MatPlotlib panels in a single frame
    """
    def __init__(self, parent=None, title ='Residual Plot Frame',
                 framesize=(850,450), panelsize=(550,450),
                 ratio=3.0, **kws):

        StackedPlotFrame.__init__(self, parent=parent, title=title,
                           framesize=framesize, **kws)
       

    def plot_residual(self, x, y1, y2, label1='Raw data', label2='Fit/theory', 
             xlabel=None, ylabel=None, show_legend=True,
             **kws):
        """plot after clearing current plot """

        panel = self.get_panel('top')
        panel.plot(x, y1, label=label1, **kws)
        panel = self.get_panel('top')
        panel.oplot(x, y2, label=label2, ylabel=ylabel, show_legend=show_legend, **kws)
        panel = self.get_panel('bottom')
        panel.plot(x, (y2-y1), ylabel='residual', show_legend=False, **kws)
        
        if xlabel is not None:
            self.xlabel = xlabel
        if self.xlabel is not None:
            self.panel_bot.set_xlabel(self.xlabel)



# This could be comparable to
# https://matplotlib.org/stable/gallery/animation/strip_chart.html

import numpy as np
from wxmplot.interactive import plot, set_data_generator

class Scope:
    def __init__(self, nmax=50, dt=0.1):
        self.dt = dt
        self.nmax = nmax
        self.t = [0]
        self.y = [0]

    def update(self):
        if len(self.t) > self.nmax:
            self.t = [0]
            self.y = [0]

        p = np.random.rand()
        v = np.random.rand() if p < 0.15 else 0
        self.y.append(v)
        self.t.append(len(self.t))
        return [(np.array(self.t)*self.dt, np.array(self.y))]

NMAX, dt = 200, 0.05
scope = Scope(nmax=NMAX, dt=dt)

plot([0], [0], xmax=dt*NMAX)
set_data_generator(scope.update, polltime=int(1000*dt))



#!/usr/bin/python
import numpy as np
import wxmplot.interactive as wi

x = np.arange(0.0,10.0,0.1)
y = np.sin(2*x)/(x+2)

win1 = wi.plot(x, y, title='Window 1', xlabel='X (mm)', win=1)

win2 = wi.plot(x, np.cos(x-4), title='Window 2', xlabel='X (mm)', win=2)

pos = win2.GetPosition()
siz = win1.GetSize()
win2.SetPosition((pos[0]+int(siz[0]*0.8), pos[1]+10))



from os import path
from tifffile import imread

import wxmplot.interactive as wi

thisdir, _ = path.split(__file__)
imgdata =  imread(path.join(thisdir, 'ceo2.tiff'))

wi.imshow(imgdata, contrast_level=0.1, colormap='coolwarm')



#!/usr/bin/env python
from numpy import linspace, sin, random
import wxmplot.interactive as wi

x = linspace(0.0, 15.0, 151)
y = 5*sin(4*x)/(x*x+6)
z = 4.8*sin(4.3*x)/(x*x+8) + random.normal(size=len(x), scale=0.05)

wi.plot(x, y, wintitle='wxmplot example',  label='reference',
        ylabel=r'$\phi(\tau)$', xlabel=r'$\tau \> \rm (ms)$')

wi.plot(x, z, label='signal', marker='+', show_legend=True)



# wxmplot documentation build configuration file
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import os
import sys
from datetime import date

from packaging.version import parse as version_parse

import wxmplot

project = 'wxmplot'
copyright = f'{date.today()}, Matthew Newville, The University of Chicago'

release = version_parse(wxmplot.__version__).public

html_title = "WXMPLOT: interactive plotting with Python"
html_short_title = "WXMPLOT: wxPython+matplotlib"

# sys.path.append(os.path.abspath(os.path.join('.')))
extensions = ['sphinx.ext.autodoc', 'sphinx.ext.mathjax',
              'sphinx.ext.napoleon', 'sphinxcontrib.video']

autoclass_content = 'both'
templates_path = ['_templates']
source_suffix = {'.rst': 'restructuredtext'}
master_doc = 'index'

exclude_trees = ['_build']
default_role = None

add_function_parentheses = True
add_module_names = False
pygments_style = 'sphinx'

html_theme_path = ['sphinx_theme']
html_theme = 'bizstyle'


html_static_path = ['_static']
html_sidebars = {'index': ['indexsidebar.html', 'searchbox.html']}
html_domain_indices = False
html_use_index = True
html_show_sourcelink = True

latex_documents = [('index', 'wxmplot.tex', 'wxmplot documentation',
                    'Matthew Newville', 'manual')]
